
from platform import system
import crocodile.toolbox as tb
from machineconfig.utils.utils import LIBRARY_ROOT


def main(verbose=True) -> str:
    repos = tb.P.home().joinpath("dotfiles/config/repos.ini")
    if repos.exists():
        repos = repos.readit()
        repo_package_list = repos.sections()
    else:
        repos = {
            "crocodile": dict(name="crocodile", url="http://github.com/thisismygitrepo/crocodile", py_package="True", path=""),
            "machineconfig": dict(name="machineconfig", url="http://github.com/thisismygitrepo/machineconfig", py_package="True", path="")
                }
        repo_package_list = repos.keys()

    local_install_repos = []
    global_packages = []
    for a_package in repo_package_list:
        try:
            a_package_path = tb.P(__import__(a_package).__file__) if repos[a_package]["py_package"] == "True" else tb.P(repos[a_package]["path"]).expanduser().absolute()
            if not a_package_path.exists():
                if verbose: print(f"Couldn't find {a_package} repo. Cloning from remote {repos[a_package]['url']} to ~/code/{a_package} ...")
                tb.Terminal().run(f"cd ~/code; git clone {repos[a_package]['url']}")
            repo = tb.install_n_import("git", "gitpython").Repo(str(a_package_path), search_parent_directories=True)
            local_install_repos.append(repo)
        except:
            if repos[a_package]["py_package"] == "False":
                continue
            global_packages.append(a_package)

    if verbose: print(f"Local install repos:"); tb.L(local_install_repos).print(); print("Global packages:"); tb.L(global_packages).print()
    sep = "\n"
    if system() == "Linux":
        program = tb.P(f"{LIBRARY_ROOT}/jobs/linux/update_essentials").read_text()
        additions = []
        for a_repo in local_install_repos:
            if "machineconfig" in a_repo.working_dir:
                an_addition = f"""
cd "{a_repo.working_dir}"
echo "{("Pulling " + str(a_repo.working_dir)).center(80, "-")}"
git reset --hard
git pull origin
chmod +x ~/scripts -R
chmod +x ~/code/machineconfig/src/machineconfig/jobs/linux -R
chmod +x ~/code/machineconfig/src/machineconfig/settings/lf_linux/exe -R
"""
                additions.append(an_addition)
            else:
                additions.append(f"""
cd "{a_repo.working_dir}"
echo "{("Pulling " + str(a_repo.working_dir)).center(80, "-")}"
{sep.join([f'git pull {remote.name}' for remote in a_repo.remotes])}
""")
        addition = "\n".join(additions)
    elif system() == "Windows":
        program = tb.P(f"{LIBRARY_ROOT}/jobs/windows/update_essentials.ps1").read_text()
        addition = "\n".join([f"""
cd "{a_repo.working_dir}"
echo "{("Pulling " + str(a_repo.working_dir)).center(80, "-")}"
{sep.join([f'git pull {remote.name}' for remote in a_repo.remotes])}
""" for a_repo in local_install_repos])
    else: raise NotImplementedError(f"System {system()} not supported")

    program = program.split("# updateBegins")[0] + addition + program.split("# updateEnds")[1]
    program += f"\npip install --upgrade {' '.join(global_packages)}\n" if len(global_packages) else ""
    return program


if __name__ == '__main__':
    pass




# bat (rust-based, modern cat)
#  cargo install bat
# colored bat
from machineconfig.utils.utils import get_latest_release


url = 'https://github.com/sharkdp/bat'


def main(version=None): get_latest_release(url, download_n_extract=True, version=version)


if __name__ == '__main__':
    main()




from machineconfig.utils.utils import get_latest_release
from platform import system
import crocodile.toolbox as tb

repo_url = tb.P(r"https://github.com/helix-editor/helix")


def main(version=None):
    if system() == 'Windows':
        from crocodile.environment import AppData
        target = AppData
        suffix = "x86_64-windows"
        compression = "zip"
        exe = get_latest_release(repo_url.as_url_str(), download_n_extract=True, suffix=suffix, delete=False, exe_name="hx", version=version)
    else:
        target = tb.P.home().joinpath(".config")
        suffix = "x86_64-linux"
        compression = 'tar.xz'
        exe = get_latest_release(repo_url.as_url_str(), download_n_extract=False, suffix=suffix, version=version)
        name = f'{repo_url[-1]}-{exe[-1]}-{suffix}.tar.xz'
        exe = exe.joinpath(name).download().unxz_untar(inplace=True)
        exe = exe.search()[0].joinpath("hx")
        exe.chmod(0o777)
        # exe.move(folder=r"/usr/local/bin", overwrite=True)
        tb.Terminal().run(f"sudo mv {exe} /usr/local/bin/").print_if_unsuccessful(desc="MOVING executable to /usr/local/bin", strict_err=True, strict_returncode=True)

    exe.parent.joinpath("runtime").move(folder=target.joinpath("helix"), overwrite=True)
    exe.parent.joinpath("contrib").move(folder=target.joinpath("helix"), overwrite=True)
    exe.parent.parent.delete(sure=True)

# as per https://github.com/helix-editor/helix/wiki/How-to-install-the-default-language-servers
# and https://spacevim.org/use-vim-as-a-python-ide/


"""
#cd ~; mkdir "tmp_asdf"; cd tmp_asdf
#latest=$(get_latest_release "helix-editor/helix")
#wget https://github.com/helix-editor/helix/releases/download/$latest/helix-$latest-x86_64-linux.tar.xz
#tar -xf *; chmod +x ./nu; sudo mv ./nu /usr/local/bin/; cd ~; rm -rdf tmp_asdf
"""


if __name__ == '__main__':
    main()




import crocodile.toolbox as tb
import machineconfig.jobs.python_windows_installers as inst
import machineconfig.jobs.python_generic_installers as gens
from machineconfig.jobs.python.python_linux_installers_all import main as linux_main


def get_cli_py_installers():
    return tb.P(inst.__file__).parent.search("*.py", filters=[lambda x: "__init__" not in str(x)]) + tb.P(gens.__file__).parent.search("*.py", filters=[lambda x: "__init__" not in str(x)])


def main():
    installers = tb.L(get_cli_py_installers())
    linux_main(installers=installers)


if __name__ == '__main__':
    main()




import numpy as np
import matplotlib.pyplot as plt
import crocodile.toolbox as tb
from platform import system


def main(version=None):
    _ = version
    # see remove paywalls and enhance YT experience by Chris Titus
    folder = r"C:\\"
    tb.P("https://github.com/iamadamdev/bypass-paywalls-chrome/archive/master.zip").download().unzip(folder=folder, content=True)
    tb.P(folder).joinpath("bypass-paywalls-chrome-master")
    return ""


if __name__ == '__main__':
    main()




import argparse
import configparser
from pathlib import Path
import random
import string
import os


def get_conn_string(sess_name):
    creds = configparser.ConfigParser()
    creds.read(Path.home().joinpath('dotfiles/creds/tmate/creds.ini'))
    sess_name = creds['sessions_names'][sess_name]
    user_name = creds['keys']['username']
    return f"{user_name}/{sess_name}@sgp1.tmate.io"


def main():
    parser = argparse.ArgumentParser(description='Tmate launcher')
    parser.add_argument("sess_name", help=f"session name", default=random.choices(list(string.digits + string.ascii_letters), k=20))
    args = parser.parse_args()
    conn_string = get_conn_string(args.sess_name)
    print(f"ssh {conn_string}")
    os.system(f"ssh {conn_string}")


if __name__ == '__main__':
    main()





import crocodile.toolbox as tb

# one can either install rufus: https://rufus.ie/en/
# however, to create bootable media with multiple OSs to choose from:

tb.P(r'https://github.com/ventoy/Ventoy/archive/refs/tags/v1.0.78.zip').download().unzip().find()()
tb.P(r'https://mirrors.layeronline.com/linuxmint/stable/21/linuxmint-21-cinnamon-64bit.iso').download(folder=tb.P.home().joinpath("Downloads/os").create())
tb.P(r'https://download.manjaro.org/kde/21.3.7/manjaro-kde-21.3.7-minimal-220816-linux515.iso').download(folder=tb.P.home().joinpath("Downloads/os").create())


if __name__ == '__main__':
    pass





from machineconfig.utils.utils import get_latest_release
import crocodile.toolbox as tb


def main(version=None):
    repo_url = tb.P(r"https://github.com/topgrade-rs/topgrade")
    _ = get_latest_release(repo_url.as_url_str(), suffix='x86_64-unknown-linux-gnu', compression='tar.gz', linux=True, strip_v=False, download_n_extract=True, version=version)


if __name__ == '__main__':
    main()




import crocodile.toolbox as tb
# import crocodile.environment as env
import machineconfig
from rich.text import Text
from rich.console import Console
from rich.panel import Panel


LIBRARY_ROOT = machineconfig.__file__.replace(tb.P.home().str.lower(), tb.P.home().str)
LIBRARY_ROOT = tb.P(LIBRARY_ROOT).parent
REPO_ROOT = LIBRARY_ROOT.parent.parent
tmp_install_dir = tb.P.tmp(folder="tmp_installers")


def display_options(msg, options: list, header="", tail="", prompt="", default=None):
    console = Console()
    if default is not None:
        assert default in options, f"Default `{default}` option not in options `{list(options)}`"
        default_msg = Text(f" <<<<-------- DEFAULT", style="bold red")
    else: default_msg = ""
    txt = Text("\n" + msg + "\n")
    for idx, key in enumerate(options):
        txt = txt + Text(f"{idx:2d} ", style="bold blue") + str(key) + (default_msg if default is not None and default == key else "") + "\n"
    txt = Panel(txt, title=header, subtitle=tail, border_style="bold red")
    console.print(txt)

    choice_idx = input(f"{prompt}\nEnter option *number* (or option name starting with space): ")
    if choice_idx.startswith(" "):
        choice_key = choice_idx.strip()
        assert choice_key in options, f"Choice `{choice_key}` not in options `{options}`"
        choice_idx = options.index(choice_key)
    else:
        if choice_idx == "":
            assert default is not None, f"Default option not available!"
            choice_idx = options.index(default)
        try:
            try:
                choice_idx = int(choice_idx)
            except ValueError:  # parsing error
                raise IndexError
            choice_key = options[choice_idx]
        except IndexError:
            # many be user forgotten to start with a dash
            if choice_idx in options:
                choice_key = choice_idx
                choice_idx = options.index(choice_idx)
            else:
                raise ValueError(f"Unknown choice. {choice_idx}")

    print(f"{choice_idx}: {choice_key}", f"<<<<-------- CHOICE MADE")
    return choice_key


def symlink(this: tb.P, to_this: tb.P, overwrite=True):
    """helper function. creates a symlink from `this` to `to_this`.
    What can go wrong?
    depending on this and to_this existence, one will be prioretized depending on overwrite value.
    True means this will potentially be overwritten (depending on whether to_this exists or not)
    False means to_this will potentially be overwittten."""
    this = tb.P(this).expanduser().absolute()
    to_this = tb.P(to_this).expanduser().absolute()
    if this.is_symlink(): this.delete(sure=True)  # delete if it exists as symblic link, not a concrete path.
    if this.exists():  # this is a problem. It will be resolved via `overwrite`
        if overwrite is True:  # it *can* be deleted, but let's look at target first.
            if to_this.exists():  # this exists, to_this as well. to_this is prioritized.
                this.append(f".orig_{tb.randstr()}", inplace=True)  # rename is better than deletion
            else: this.move(path=to_this)  # this exists, to_this doesn't. to_this is prioritized.
        elif overwrite is False:  # don't sacrefice this, sacrefice to_this.
            if to_this.exists(): this.move(path=to_this, overwrite=True)  # this exists, to_this as well, this is prioritized.   # now we are readly to make the link
            else: this.move(path=to_this)  # this exists, to_this doesn't, this is prioritized.
    else:  # this doesn't exist.
        if not to_this.exists(): to_this.touch()  # we have to touch it (file) or create it (folder)
    try: tb.P(this).symlink_to(to_this, verbose=True, overwrite=True)
    except Exception as ex: print(f"Failed at linking {this} ==> {to_this}.\nReason: {ex}")


def find_move_delete_windows(downloaded, tool_name=None, delete=True):
    if tool_name is not None and ".exe" in tool_name: tool_name = tool_name.replace(".exe", "")
    if downloaded.is_file():
        exe = downloaded
    else:
        exe = downloaded.search("*.exe", r=True)[0] if tool_name is None else downloaded.search(f"{tool_name}.exe", r=True)[0]
    exe.move(folder=tb.P.get_env().WindowsApps, overwrite=True)  # latest version overwrites older installation.
    if delete: downloaded.delete(sure=True)
    return exe


def find_move_delete_linux(downloaded, tool_name, delete=True):
    if downloaded.is_file():
        exe = downloaded
    else:
        res = downloaded.search(f"*{tool_name}*", folders=False, r=True)
        if len(res) == 1: exe = res[0]
        else: exe = downloaded.search(tool_name, folders=False, r=True)[0]
    print(f"MOVING file `{repr(exe)}` to '/usr/local/bin'")
    exe.chmod(0o777)
    # exe.move(folder=r"/usr/local/bin", overwrite=False)
    tb.Terminal().run(f"sudo mv {exe} /usr/local/bin/").print_if_unsuccessful(desc="MOVING executable to /usr/local/bin", strict_err=True, strict_returncode=True)
    if delete: downloaded.delete(sure=True)
    return None


def get_latest_release(repo_url, download_n_extract=False, suffix="x86_64-pc-windows-msvc", file_name=None, tool_name=None, exe_name=None, delete=True, strip_v=False, linux=False, compression=None, sep="-", version=None):
    console = Console()
    print("\n\n\n")
    print("Installing...")
    # with console.status("Installing..."):  # makes troubles on linux when prompt asks for password to move file to /usr/bin

    if version is None:
        import requests  # https://docs.github.com/en/repositories/releasing-projects-on-github/linking-to-releases
        latest_version = requests.get(str(repo_url) + "/releases/latest").url.split("/")[-1]  # this is to resolve the redirection that occures: https://stackoverflow.com/questions/36070821/how-to-get-redirect-url-using-python-requests
    else: latest_version = version

    download_link = tb.P(repo_url + "/releases/download/" + latest_version)

    version = download_link[-1]
    version = str(version).replace("v", "") if strip_v else str(version)
    tool_name = tool_name or tb.P(repo_url)[-1]
    console.rule(f"Installing {tool_name} version {version}")
    tb.P.home().joinpath(f"tmp_results/cli_tools_installers/versions/{tool_name}").create(parents_only=True).write_text(version)

    if not download_n_extract: return download_link
    if download_n_extract and not linux:
        if file_name is None:  # it is not constant, so we compile it from parts as follows:
            file_name = f'{tool_name}{sep}{version}{sep}{suffix}.{compression or "zip"}'
        print("Downloading", download_link.joinpath(file_name))
        downloaded = download_link.joinpath(file_name).download(folder=tmp_install_dir).unzip(inplace=True, overwrite=True)
        return find_move_delete_windows(downloaded, exe_name or tool_name, delete)
    elif download_n_extract and linux:
        if file_name is None:  # it is not constant, so we compile it from parts as follows:
            file_name = f'{tool_name}-{version}-{suffix}.{compression or "tar.gz"}'
        download_link = download_link.joinpath(file_name)
        print("Downloading", download_link)
        downloaded = download_link.download(folder=tmp_install_dir)
        if "tar.gz" in download_link: downloaded = downloaded.ungz_untar(inplace=True)
        elif "zip" in download_link: downloaded = downloaded.unzip(inplace=True)
        elif "tar.xz" in download_link: downloaded = downloaded.unxz_untar(inplace=True)
        else: pass  # no compression.
        return find_move_delete_linux(downloaded, exe_name or tool_name, delete)

    # console.rule(f"Completed Installation")
    # return res


def get_shell_script_executing_pyscript(python_file, func=None, system=None, ve_name="ve"):
    if system == "Windows":
        return f"""
~/scripts/activate_ve
python {python_file} {func if func is not None else ""}
"""
    else: return f"""
source ~/scripts/activate_ve
python {python_file} {func if func is not None else ""}
deactivate
"""


if __name__ == '__main__':
    pass




from machineconfig.utils.utils import get_latest_release, find_move_delete_linux
from platform import system
import crocodile.toolbox as tb

repo_url = tb.P(r"https://github.com/neovim/neovim")
release = get_latest_release(repo_url.as_url_str(), download_n_extract=False)

if system() == 'Windows':
    release.joinpath("nvim-win64.msi").download()()
else:
    release.joinpath("nvim-linux64.tar.gz").download().ungz_untar(inplace=True)

if __name__ == '__main__':
    pass





# wget https://dystroy.org/broot/download/x86_64-linux/broot
# # broot is an fzf variant. It's excellent for viewing folder structure and layered search.
# # broot doesn't search aribtrarily deep and it also avoids git folders.
# chmod +x broot
# sudo mv ./broot /usr/local/bin/

import crocodile.toolbox as tb
from rich.console import Console
from platform import system
from machineconfig.utils.utils import get_latest_release


if system() == 'Linux':
    url = r'https://dystroy.org/broot/download/x86_64-linux/broot'
elif system() == 'Windows':
    url = r'https://dystroy.org/broot/download/x86_64-pc-windows-gnu/broot.exe'
else:
    raise Exception(f"Unsupported OS: {system()}")


def main(version=None):
    print("\n\n\n")
    console = Console()
    console.rule("Installing Broot")
    _ = get_latest_release("https://github.com/Canop/broot", version=version)

    if system() == "Windows":
        p = tb.P(url).download()
        p.move(folder=tb.get_env().WindowsApps, overwrite=True)
    else:
        p = tb.P(url).download()
        p.chmod(0o777)  # p.move(folder=r'/usr/local/bin/', overwrite=True) Permission Error
        tb.Terminal().run(f"sudo mv {p} /usr/local/bin/").print_if_unsuccessful(desc="MOVING executable to /usr/local/bin", strict_err=True, strict_returncode=True)

    console.rule("Completed Installation")


if __name__ == '__main__':
    main()





from crocodile.comms.gdrive import GDriveAPI
# from crocodile.file_management import P
import argparse


def main(google_account, project, file, encrypt_first, z, relative_to_home, remote_dir, share, key, pwd):
    api = GDriveAPI(account=google_account, project=project)

    if share: res = api.upload_and_share(local_path=file, rel2home=relative_to_home,
                                         zip_first=z, encrypt_first=encrypt_first, key=key, pwd=pwd)
    else: res = api.upload(local_path=file, remote_dir=remote_dir, rel2home=relative_to_home,
                           zip_first=z, encrypt_first=encrypt_first, key=key, pwd=pwd)  # , recursive, zipFirst)
    print(res)


def args_parser():
    parser = argparse.ArgumentParser(description='FTP client')

    # positional argument
    parser.add_argument("file", help="file/folder path.", default="")
    # FLAGS
    # parser.add_argument("--recursive", "-r", help="Send recursively.", action="store_true")  # default is False
    parser.add_argument("--zip_first", "-z", help="Zip before sending.", action="store_true")  # default is False
    parser.add_argument("--encrypt_first", "-e", help="Encrypt before sending.", action="store_true")  # default is False
    parser.add_argument("--relative_to_home", "-R", help="Zip before sending.", action="store_true")  # default is False
    parser.add_argument("--share", "-s", help="Share file.", action="store_true")
    # optional argument
    parser.add_argument("--remote_dir", "-d", help="Remote directory to send to.", default="")
    parser.add_argument("--google_account", "-a", help="Google Account.", default=None)
    parser.add_argument("--project", "-P", help="Project Name", default=None)
    parser.add_argument("--key", "-k", help="Key for encryption", default=None)
    parser.add_argument("--pwd", "-p", help="Password for encryption", default=None)

    args = parser.parse_args()
    main(google_account=args.google_account, project=args.project, file=args.file, z=args.zip_first, relative_to_home=args.relative_to_home, remote_dir=args.remote_dir, share=args.share, key=args.key, pwd=args.pwd, encrypt_first=args.encrypt_first)


if __name__ == "__main__":
    args_parser()





from crocodile.comms.gdrive import GDriveAPI
# from crocodile.file_management import P
import argparse


def main(google_account, project, file, unzip, relative_to_home, decrypt, key, pwd, local_dir):
    api = GDriveAPI(account=google_account, project=project)

    if "http" in file: path = api.download(furl=file, local_dir=local_dir, rel2home=relative_to_home)  # , recursive, zipFirst)
    else:
        file = file
        path = api.download(fpath=file, local_dir=local_dir, rel2home=relative_to_home, decrypt=decrypt, unzip=unzip, key=key, pwd=pwd)  # , recursive, zipFirst)
    print(path)


def arg_parser():
    parser = argparse.ArgumentParser(description='FTP client')

    # positional argument
    parser.add_argument("file", help="file/folder path.", default="")
    # FLAGS
    # parser.add_argument("--recursive", "-r", help="Send recursively.", action="store_true")  # default is False
    parser.add_argument("--relative_to_home", "-R", help="Download to a directory to make it relative to home.", action="store_true")  # default is False
    parser.add_argument("--decrypt", "-e", help="decrypt file.", action="store_true")  # default is False
    parser.add_argument("--unzip", "-z", help="unzip file.", action="store_true")  # default is False
    # optional argument
    parser.add_argument("--local_dir", "-d", help="Local directory to put the file in.", default="")
    parser.add_argument("--google_account", "-a", help="Google Account.", default=None)
    parser.add_argument("--project", "-P", help="Project Name", default=None)
    parser.add_argument("--key", "-k", help="Key for decryption", default=None)
    parser.add_argument("--pwd", "-p", help="Password for decryption", default=None)

    args = parser.parse_args()
    main(google_account=args.google_account, project=args.project, file=args.file,
         unzip=args.unzip, relative_to_home=args.relative_to_home, decrypt=args.decrypt, key=args.key, pwd=args.pwd, local_dir=args.local_dir)


if __name__ == "__main__":
    arg_parser()




# search for words in files.
from machineconfig.utils.utils import get_latest_release

url = r'https://github.com/Genivia/ugrep'


def main(version=None):
    f = get_latest_release(url, version=version).joinpath('ugrep.exe').download()
    f.move(folder=f.get_env().WindowsApps, overwrite=True)


if __name__ == '__main__':
    main()




import crocodile.toolbox as tb
from rich.console import Console

url = r'https://download.sysinternals.com/files/ZoomIt.zip'


def main(version=None):
    _ = version
    print("\n\n\n")
    console = Console()
    console.rule("Installing ZoomIt")
    folder = tb.P(url).download(tb.P.home().joinpath('Downloads')).unzip(inplace=True)
    folder.joinpath('ZoomIt.exe').move(folder=tb.get_env().WindowsApps, overwrite=True)
    folder.delete(sure=True)
    console.rule("Completed Installation")


if __name__ == '__main__':
    main()





from platform import system
# import subprocess
import crocodile.toolbox as tb
from machineconfig.utils.utils import LIBRARY_ROOT, display_options


def main(program_name=None):
    if system() == "Windows":
        from machineconfig.jobs.python.python_windows_installers_all import get_cli_py_installers
    else:
        from machineconfig.jobs.python.python_linux_installers_all import get_cli_py_installers
    installers = get_cli_py_installers()
    default = tb.P("all")
    installers.list.insert(0, default)
    options = list(installers.stem)
    options.sort()

    if program_name is None:
        program_name = display_options(msg="", options=options, header="CHOOSE DEV APP", default=str(default))

    if program_name == "all":
        # program_linux = f"source {LIBRARY_ROOT}/setup_linux/devapps.sh"
        # program_windows = f"{LIBRARY_ROOT}/setup_windows/devapps.ps1"
        if system() == "Linux":
            from machineconfig.jobs.python.python_linux_installers_all import main
            main()
        elif system() == "Windows":
            from machineconfig.jobs.python.python_windows_installers_all import main
            main()
        else:
            raise NotImplementedError(f"System {system()} not supported")
        program = ""

    else:
        idx = installers.stem.list.index(program_name)
        program = installers[idx].readit()['main']()  # finish the task
        if program is None: program = "echo 'Finished Installation'"  # write an empty program
    return program


if __name__ == '__main__':
    pass




import argparse
from crocodile.toolbox import SSH
import crocodile.toolbox as tb


def main():
    parser = argparse.ArgumentParser(description='FTP client')

    parser.add_argument("machine", help=f"machine ssh address", default="")
    parser.add_argument("file", help="file/folder path.", default="")
    # FLAGS
    parser.add_argument("--recursive", "-r", help="Send recursively.", action="store_true")  # default is False
    parser.add_argument("--zipFirst", "-z", help="Zip before sending.", action="store_true")  # default is False

    parser.add_argument("-d", "--destination", help=f"destination folder", default=None)

    args = parser.parse_args()
    import paramiko
    try:
        ssh = SSH(rf'{args.machine}')
    except paramiko.ssh_exception.AuthenticationException:
        print("Authentication failed, trying manually:")
        print(f"Caution: Ensure that username is passed appropriately as this exception only handles password.")
        import getpass
        pwd = getpass.getpass()
        ssh = SSH(rf'{args.machine}', pwd=pwd)

    received_file = ssh.copy_to_here(source=args.file, target=args.destination, z=args.zipFirst, r=args.recursive)
    # ssh.print_summary()

    if tb.P(args.file).is_dir(): print(f"Use: cd {repr(tb.P(args.file).expanduser())}")
    elif received_file is not None: print(f"Received: {repr(received_file.parent), repr(received_file)}")


if __name__ == '__main__':
    main()




"""
doesn't have --version
"""

from machineconfig.utils.utils import get_latest_release, find_move_delete_linux, find_move_delete_windows
from platform import system


url = "https://github.com/netxs-group/vtm"


def main():
    link = get_latest_release(url, download_n_extract=False)
    if system() == 'Windows':
        downloaded = link.joinpath("vtm_windows_amd64.zip").download().unzip(inplace=True)
        find_move_delete_windows(downloaded, "vtm", delete=True)
    else:
        downloaded = link.joinpath("vtm_linux_amd64.tar.gz").download().ungz_untar(inplace=True)
        find_move_delete_linux(downloaded, tool_name="vtm")
    return ""


if __name__ == '__main__':
    main()




from platform import system
from machineconfig.utils.utils import get_latest_release, tb, find_move_delete_linux, find_move_delete_windows


def main(version=None):
    # =================================================== python's qr code to create qrcode on terminal from simple text.
    tb.Terminal().run("pip install qrcode", shell="pwsh")

    # =================================================== Go's qrcp to allow file transfer between computer and phone.
    url = get_latest_release("https://github.com/claudiodangelis/qrcp", download_n_extract=False, version=version)
    if system() == "Windows":
        downloaded = url.joinpath(f"qrcp_{url[-1].str.replace('v', '')}_Windows_x86_64.tar.gz").download().ungz_untar(inplace=True)
        find_move_delete_windows(downloaded, "qrcp")
    else:
        downloaded = url.joinpath(f"qrcp_{url[-1].str.replace('v', '')}_linux_x86_64.tar.gz").download().ungz_untar(inplace=True)
        find_move_delete_linux(downloaded, "qrcp")


if __name__ == '__main__':
    main()




import crocodile.toolbox as tb
import platform
import machineconfig
from rich.panel import Panel
from rich.console import Console
# from rich.text import Text
from rich.syntax import Syntax

system = platform.system()
lib_root = tb.P(machineconfig.__file__).parent


def main():
    console = Console()
    print("\n\n")
    console.rule("Existing Python versions", style="bold red")
    if system == "Windows":
        tb.P.get_env().Path.search("python.exe").reduce()[1:].print()
    else:
        tb.L(set(tb.P.get_env().Path.search("python3*").reduce().filter(lambda x: not x.is_symlink() and "-" not in x))).print()
    print("\n\n")
    console.rule(f"Existing virtual environments")
    ves = tb.P.home().joinpath("venvs").search("*", files=False).apply(lambda a_ve: (a_ve.name, a_ve.joinpath("pyvenv.cfg").read_text()))
    ves.apply(lambda a_ve: console.print(Panel(a_ve[1], title=a_ve[0], style="bold blue")))
    # ves.apply(lambda a_ve: tb.S(a_ve[1]).print(as_config=True, title=a_ve[0]))

    dotted_py_version = input("Enter python version (3.9): ") or "3.9"
    env_name = input("Enter virtual environment name (tst): ") or "tst"
    repos = input("Install essential repos? (y/[n]): ") or "n"

    env_path = tb.P.home().joinpath("venvs", env_name)
    if env_path.exists():
        env_path.delete(sure=True)

    scripts = lib_root.joinpath(f"setup_{system.lower()}/ve.{'ps1' if system == 'Windows' else 'sh'}").read_text()
    variable_prefix = "$" if system == "Windows" else ""
    scripts = tb.modify_text(txt_raw=scripts, txt_search="ve_name=", txt_alt=f"{variable_prefix}ve_name='{env_name}'", replace_line=True, strict=True)
    scripts = tb.modify_text(txt_raw=scripts, txt_search="py_version=", txt_alt=f"{variable_prefix}py_version='{dotted_py_version.replace('.', '') if system == 'Windows' else dotted_py_version}'", replace_line=True, strict=True)

    if repos == "y":
        text = lib_root.joinpath(f"setup_{system.lower()}/repos.{'ps1' if system == 'Windows' else 'sh'}").read_text()
        text = tb.modify_text(txt_raw=text, txt_search="ve_name=", txt_alt=f"{variable_prefix}ve_name='{env_name}'", replace_line=True)
        scripts += text

    print("\n")
    console.print(Panel(Syntax(scripts, lexer="ps1" if system == "Windows" else "sh"), title="Script to create virtual environment..."), style="bold red")

    return scripts


if __name__ == '__main__':
    pass




# fd (rust)
# cargo install fd-find
# search for files/folder names only. Result can be fed to fzf for further filtering. For that use: fd | fzf. one can also make fd the searcher in fzf

url = 'https://github.com/sharkdp/fd'

def main(version=None):
    from machineconfig.utils.utils import get_latest_release
    get_latest_release(url, download_n_extract=True, version=version)


if __name__ == '__main__':
    main()




import argparse
import configparser
from pathlib import Path
import random
import string
import os


# TODO make it automatically start a zellij session and pick appropraite tmate session matching machine name


def main():
    creds = configparser.ConfigParser()
    creds.read(Path.home().joinpath('dotfiles/creds/tmate/creds.ini'))

    parser = argparse.ArgumentParser(description='Tmate launcher')
    random_sess = random.choices(list(string.digits + string.ascii_letters), k=20)
    _ = random_sess
    parser.add_argument("sess_name", help=f"session name (new only with random string will be chosen if not passed)", default=None)

    args = parser.parse_args()

    sess_name = creds['sessions_names'][args.sess_name]
    api_key = creds['keys']['api_key']
    res = f"tmate -a ~/.ssh/authorized_keys -k {api_key} -n {sess_name} -F"
    os.system(res)


if __name__ == '__main__':
    main()




from crocodile.toolbox import Log, install_n_import, Scheduler, P
import time
from datetime import datetime


def pomodoro(work=25, rest=5, repeats=4):
    logger = Log(name="pomodoro", file=False, stream=True)
    def loop(sched):
        speak("Alright, time to start working..."); start = datetime.now(); _ = sched
        while (diff := work - ((datetime.now() - start).seconds / 60)) > 0: logger.debug(f"Keep working. Time Left: {round(diff)} minutes"); time.sleep(60 * 1)
        speak("Now, its time to take a break."); start = datetime.now()
        while (diff := rest - ((datetime.now() - start).seconds / 60)) > 0: logger.critical(f"Keep Resting. Time Left: {round(diff)} minutes"); time.sleep(60 * 1)
    def speak(txt):
        install_n_import("gtts").gTTS(txt, lang='en', tld='com.au').save(tmp := P.tmpfile(suffix=".mp3")); time.sleep(0.5)
        pyglet = install_n_import("pyglet"); pyglet.resource.path = [tmp.parent.str]; pyglet.resource.reindex(); pyglet.resource.media(tmp.name).play()
    def beep(duration=1, frequency=3000):
        try: import winsound
        except ImportError: __import__("os").system('beep -f %s -l %s' % (frequency, 1000 * duration))
        else: winsound.Beep(frequency, 1000 * duration)
    return Scheduler(routine=loop, max_cycles=repeats, logger=logger, wait="0.1m").run()


# def main():
#     parser = argparse.ArgumentParser(description='FTP client')
#
#     parser.add_argument("machine", help=f"machine ssh address", default="")
#     parser.add_argument("file", help="file/folder path.", default="")
#     # FLAGS
#     parser.add_argument("--recursive", "-r", help="Send recursively.", action="store_true")  # default is False
#     parser.add_argument("--zipFirst", "-z", help="Zip before sending.", action="store_true")  # default is False
#
#     args = parser.parse_args()


if __name__ == '__main__':
    pomodoro()





from machineconfig.utils.utils import get_latest_release
import crocodile.toolbox as tb


def main(version=None):
    repo_url = tb.P(r"https://github.com/atanunq/viu")
    release = get_latest_release(repo_url.as_url_str(), version=version)
    exe = release.joinpath(f"viu").download()
    exe.chmod(0o777)
    # exe.move(folder=r"/usr/local/bin", overwrite=False)
    tb.Terminal().run(f"sudo mv {exe} /usr/local/bin/").print_if_unsuccessful(desc="MOVING executable to /usr/local/bin", strict_err=True, strict_returncode=True)


if __name__ == '__main__':
    pass





import crocodile.toolbox as tb
tm = tb.Terminal()


def commit_one(path, mess="auto_commit_" + tb.randstr()):
    return f'''
echo "{("Committing " + str(path)).center(80, "-")}"
cd {path}; git add .; git commit -am "{mess}"'
echo ""
echo ""
'''


def push_one(path):
    remotes = tm.run(f"cd {path}; git remote", shell="powershell").op.split("\n")
    cmds = []
    join = '\n'
    for remote in remotes:
        if remote != "": cmds.append(f'cd {path}; git push {remote}')
    return f"""
echo "{("Pushing " + str(path)).center(80, "-")}"
{join.join(cmds)}
echo ""
echo ""
"""


def pull_one(path):
    remotes = tm.run(f"cd {path}; git remote", shell="powershell").op.split("\n")
    cmds = []
    join = '\n'
    for remote in remotes:
        if remote != "": cmds.append(f'cd {path}; git pull {remote}')
    return f"""
echo "{("Pulling " + str(path)).center(80, "-")}"
{join.join(cmds)}
echo ""
echo ""
"""


def commit_all():
    q = tb.P.home().joinpath("code").search("*").apply(lambda x: commit_one(x))  # reduce(lambda x, y: x + "\n\n" + y)[0]
    return "\n\n".join(q)

def push_all(): return "\n\n".join(tb.P.home().joinpath("code").search("*").apply(lambda x: push_one(x)))
def pull_all(): return "\n\n".join(tb.P.home().joinpath("code").search("*").apply(lambda x: pull_one(x)))


if __name__ == '__main__':
    pass




import argparse
import crocodile.toolbox as tb
from machineconfig.profile.create import symlink
from machineconfig.utils.utils import LIBRARY_ROOT, REPO_ROOT


def main():
    parser = argparse.ArgumentParser(description='FTP client')

    parser.add_argument("file", help="file/folder path.", default="")
    # FLAGS
    parser.add_argument("--overwrite", "-o", help="Overwrite.", action="store_true")  # default is False
    # optional
    parser.add_argument("-d", "--dest", help=f"destination folder", default="")

    args = parser.parse_args()
    orig_path = tb.P(args.file).expanduser().absolute()
    if args.dest == "":
        if "Local" in orig_path: junction = orig_path.split(at="Local", sep=-1)[1]
        elif "Roaming" in orig_path: junction = orig_path.split(at="Roaming", sep=-1)[1]
        elif ".config" in orig_path: junction = orig_path.split(at=".config", sep=-1)[1]
        else: junction = orig_path.rel2home()
        new_path = REPO_ROOT.joinpath(junction)
    else: new_path = tb.P(args.dest).expanduser().absolute().create().joinpath(orig_path.name)

    symlink(this=orig_path, to_this=new_path, overwrite=args.overwrite)

    print(f"Map completed. To enshrine this mapping, add the following line to the mapper.toml file:")
    print(f"nano {LIBRARY_ROOT}/symlinks/mapper.toml")
    print(f"""
[{new_path.parent.name}]
{orig_path.trunk} = {{ this = '{orig_path.collapseuser().as_posix()}', to_this = '{new_path.collapseuser().as_posix()}' }}
""")


if __name__ == '__main__':
    main()




from platform import system
from machineconfig.utils.utils import get_latest_release, tb, find_move_delete_linux, find_move_delete_windows


def main(version=None):
    # =================================================== Rust's qrscan to allow computers to scan qr code from webcam.
    if system() == 'Windows': _ = get_latest_release('https://github.com/sayanarijit/qrscan', download_n_extract=True, strip_v=True, version=version)
    else: _ = get_latest_release(repo_url='https://github.com/sayanarijit/qrscan', download_n_extract=True, linux=True, strip_v=True, version=version, suffix="x86_64-unknown-linux-gnu", compression="tar.gz")


if __name__ == '__main__':
    main()




# rg (rust)
# cargo install ripgrep
# used by lvim and spacevim
from machineconfig.utils.utils import get_latest_release

url = 'https://github.com/BurntSushi/ripgrep'


def main(version=None): get_latest_release(url, download_n_extract=True, exe_name='rg', version=version)


if __name__ == '__main__':
    main()





from platform import system
# import subprocess
import crocodile.toolbox as tb
from machineconfig.utils.utils import display_options
from enum import Enum


PROGRAM_PATH = tb.P.tmp().joinpath("shells/python_return_command") + (".ps1" if system() == "Windows" else ".sh")
# needs to be synced with devops.sh and devops.ps1


class Options(Enum):
    update         = 'UPDATE essential repos'
    devaps         = 'DEVAPPS install'
    ve             = 'VE install'
    sym_path_shell = 'SYMLINKS, PATH & SHELL PROFILE'
    sym_new        = 'SYMLINKS new'
    ssh_add_pubkey  = 'SSH add pub key to this machine'
    ssh_add_id     = 'SSH add identity (private key) to this machine'
    ssh_use_pair   = 'SSH use key pair to connect two machines'
    ssh_setup      = 'SSH setup'
    ssh_setup_wsl  = 'SSH setup wsl'
    repos_pull     = 'REPOS pull all'
    repos_commit   = 'REPOS commit all'
    repos_push     = 'REPOS push all'
    backup         = 'BACKUP & RETRIEVE'


def main():
    PROGRAM_PATH.delete(sure=True, verbose=False)
    options = [op.value for op in Options]
    choice_key = display_options(msg="", options=options, header="DEVOPS", default=options[0])

    if choice_key == Options.update.value:
        import machineconfig.scripts.python.devops_update_repos as helper
        program = helper.main()

    elif choice_key == Options.ve.value:
        from machineconfig.jobs.python.python_ve_installer import main
        program = main()

    elif choice_key == Options.devaps.value:
        from machineconfig.scripts.python.devops_devapps_install import main
        program = main()

    elif choice_key == Options.sym_new.value:
        from machineconfig.jobs.python.python_ve_symlink import main
        program = main()

    elif choice_key == Options.sym_path_shell.value:
        from machineconfig.profile.create import main
        main()
        program = "echo 'done with symlinks'"

    elif choice_key == Options.ssh_add_pubkey.value:
        from machineconfig.scripts.python.devops_add_ssh_key import main
        program = main()

    elif choice_key == Options.ssh_use_pair.value:
        raise NotImplementedError

    elif choice_key == Options.ssh_add_id.value:  # so that you can SSH directly withuot pointing to identity key.
        from machineconfig.scripts.python.devops_add_identity import main
        program = main()

    elif choice_key == Options.ssh_setup.value:
        program_windows = f"""Invoke-WebRequest https://raw.githubusercontent.com/thisismygitrepo/machineconfig/main/src/machineconfig/setup_windows/openssh_all.ps1 | Invoke-Expression  # https://github.com/thisismygitrepo.keys"""
        program_linux = f"""curl https://raw.githubusercontent.com/thisismygitrepo/machineconfig/main/src/machineconfig/setup_linux/openssh_all.sh | sudo bash  # https://github.com/thisismygitrepo.keys"""
        program = program_linux if system() == "Linux" else program_windows

    elif choice_key == Options.ssh_setup_wsl.value:
        program = f"""curl https://raw.githubusercontent.com/thisismygitrepo/machineconfig/main/src/machineconfig/setup_linux/openssh_wsl.sh | sudo bash"""

    elif choice_key == Options.backup.value:
        from machineconfig.scripts.python.devops_backup_retrieve import main
        program = main()

    elif choice_key == Options.repos_pull.value:
        from machineconfig.jobs.python.repos import pull_all
        program = pull_all()

    elif choice_key == Options.repos_push.value:
        from machineconfig.jobs.python.repos import push_all
        program = push_all()

    elif choice_key == Options.repos_commit.value:
        from machineconfig.jobs.python.repos import commit_all
        program = commit_all()

    else:
        raise ValueError(f"Unimplemented choice: {choice_key}")

    print(f"Executing {PROGRAM_PATH}")
    if system() == 'Windows':
        PROGRAM_PATH.create(parents_only=True).write_text(program)
    else:
        PROGRAM_PATH.create(parents_only=True).write_text(f"{program}")


if __name__ == "__main__":
    main()




import crocodile.toolbox as tb
import sys
from crocodile.msc.odds import capture_from_webcam

img_path = capture_from_webcam(show=False, wait=False, save=True)
# img_rgb = cv2.cvtColor(img_cv, cv2.COLOR_BGR2RGB)
# import cv2
# img_cv = cv2.imread(r'C:\Users\alex\Downloads\name.jpg')

# sys.path.insert(0, tb.P.home().joinpath("AppData/Local/Tesseract-OCR").str)
# import pytesseract
# print(pytesseract.image_to_string(img_cv))

q = tb.Terminal().run(f"cd ~/AppData/Local/Tesseract-OCR; pytesseract '{img_path}'", shell="pwsh").capture().op




from machineconfig.utils.utils import get_latest_release, find_move_delete_linux

url = "https://github.com/cantino/mcfly"


def main(version=None):
    get_latest_release(url, suffix="x86_64-unknown-linux-musl", download_n_extract=True, linux=True, compression="tar.gz", version=version)


if __name__ == '__main__':
    main()





from machineconfig.utils.utils import get_latest_release, find_move_delete_linux, find_move_delete_windows
# import crocodile.toolbox as tb
import platform


def main(version=None):
    url = get_latest_release("https://github.com/extrawurst/gitui", version=version)
    if platform.system() == "Linux":
        download = url.joinpath(f"gitui-linux-musl.tar.gz").download().ungz_untar(inplace=True)
        find_move_delete_linux(downloaded=download, tool_name="gitui")
    elif platform.system() == "Windows":
        download = url.joinpath(f"gitui-win.tar.gz").download().ungz_untar(inplace=True)
        find_move_delete_windows(downloaded=download, tool_name="gitui.exe")
    else:
        raise Exception(f"Not supported OS for gitui {platform.system()}")


if __name__ == '__main__':
    main()




import crocodile.toolbox as tb
import argparse


def arg_parser():
    parser = argparse.ArgumentParser(description='Cloud Download CLI.')

    # positional argument
    parser.add_argument("cloud", help="rclone cloud profile name.")
    parser.add_argument("file", help="file/folder path to be received.", default="")
    # FLAGS
    parser.add_argument("--recursive", "-r", help="Send recursively.", action="store_true")  # default is False
    parser.add_argument("--decrypt", "-e", help="Decrypt after receiving.", action="store_true")  # default is False
    parser.add_argument("--unzip", "-z", help="unzip after receiving.", action="store_true")  # default is False
    parser.add_argument("--overwrite", "-o", help="Overwrite existing file.", action="store_true")  # default is False
    # optional argument
    parser.add_argument("--remote_dir", "-d", help="Remote directory to send to.", default="")
    parser.add_argument("--relative_to_home", "-R", help="Relative to `myhome` folder", action="store_true")  # default is False
    parser.add_argument("--key", "-k", help="Key for encryption", default=None)
    parser.add_argument("--pwd", "-p", help="Password for encryption", default=None)

    args = parser.parse_args()
    tb.P(args.file).from_cloud(cloud=args.cloud, unzip=args.unzip, decrypt=args.decrypt, overwrite=args.overwrite, pwd=args.pwd, key=args.key)


if __name__ == "__main__":
    arg_parser()




# LF (glang)
from machineconfig.utils.utils import get_latest_release
import crocodile.toolbox as tb
import platform

url = 'https://github.com/gokcehan/lf'


def main(version=None):

    if platform.system() == 'Windows':
        # tb.Terminal().run("nu -c 'ps | where name == lf.exe | each { |it| kill $it.pid --force}'", shell="pwsh").print()
        tb.L(tb.install_n_import("psutil").process_iter()).filter(lambda x: x.name() == 'lf.exe').apply(lambda x: x.kill())
        get_latest_release(url, file_name='lf-windows-amd64.zip', download_n_extract=True, version=version)
    elif platform.system() == 'Linux':
        tb.Terminal().run("killall lf")
        get_latest_release(url, file_name='lf-linux-amd64.tar.gz', download_n_extract=True, linux=True, compression="tar.gz", version=version)
    else: raise NotImplementedError(f"Platform {platform.system()} not supported.")

    return f""


if __name__ == '__main__':
    main()




from machineconfig.utils.utils import get_latest_release, find_move_delete_linux
from platform import system
import crocodile.toolbox as tb

repo_url = tb.P(r"https://github.com/dandavison/delta")
# from https://github.com/dandavison/delta#configuration
config_patch = """
[core]
    pager = delta

[interactive]
    diffFilter = delta --color-only --features=interactive

[delta]
    features = decorations
    side-by-side = true

[delta "interactive"]
    keep-plus-minus-markers = false

[delta "decorations"]
    commit-decoration-style = blue ol
    commit-style = raw
    file-style = omit
    hunk-header-decoration-style = blue box
    hunk-header-file-style = red
    hunk-header-line-number-style = "#067a00"
    hunk-header-style = file line-number syntax
"""

def main(version=None):
    if system() == 'Windows':
        # from crocodile.environment import AppData
        # target = AppData
        suffix = "x86_64-pc-windows-msvc"
        _ = get_latest_release(repo_url.as_url_str(), suffix=suffix, download_n_extract=True, delete=True, strip_v=True, version=version)
    else:
        release = get_latest_release(repo_url.as_url_str(), version=version)
        path = release.joinpath(f"delta-{str(release[-1]).replace('v', '')}-x86_64-unknown-linux-gnu.tar.gz").download()
        downloaded = path.ungz_untar(inplace=True)
        _ = find_move_delete_linux(downloaded, "delta", delete=True)

    config_path = tb.P.home().joinpath(".gitconfig")
    if config_path.exists():
        config = config_path.read_text()
        if config_patch in config: pass
        else: config_path.write_text(config + "\n" + config_patch)
    else:
        config_path.delete(sure=True)  # delete a bad symlink if any
        config_path.write_text(config_patch)


if __name__ == '__main__':
    main()
    # pass






import crocodile.toolbox as tb
from machineconfig.utils.utils import display_options


descriptive_themes = ["markbull", "peru", "mojada", "festivetech", "sorin", "agnosterplus", "blueish",
                      "thecyberden", "plague", "kali", "fish", "ys", "slim", "paradox", "aliens", "atomicBit"]


def main(new_theme=None):
    """This is a helper for a powershell script.
    run this function to interactively choose a style. Optionally, inpsect the themes of oh my posh and select one:
    """
    import os
    themes_path = tb.P(os.environ["POSH_THEMES_PATH"])
    # current_theme = tb.P(os.environ["POSH_THEME"]).trunk
    profile = tb.Terminal().run("$profile", shell="pwsh").op2path()
    current_theme = tb.P(tb.L(profile.read_text().split(" ")).filter(lambda x: ".omp.json" in x)[0]).expanduser().absolute().trunk

    if new_theme == "manual":
        tb.P("https://ohmyposh.dev/docs/themes").start()  # replace ~/jan... with full path to theme. use: start $profile
        new_theme = input(f"A chrome tab with styles is opened, choose one and put its name here: [jandedobbeleer] ")
    if new_theme == "show":
        __import__("os").system("Write-Host Get-PoshThemes")
        return ""
    if new_theme is None:
        themes = themes_path.search().apply(lambda x: x.trunk)
        themes.list.sort()
        tail = ""
        new_theme = display_options(msg=f"Choose a theme number from the list above: ", tail=tail, options=list(themes) + ["suprise me"], default="suprise me",
                                    prompt=f"Recommended descriptive ones are {descriptive_themes}")
        if new_theme == "suprise me": new_theme = themes.sample()[0]
    print("Current Theme:", current_theme)
    print("New theme: ", new_theme)
    profile.modify_text(txt_search=current_theme, txt_alt=new_theme, replace_line=False)


if __name__ == '__main__':
    pass




# gives examples of how to use a command, its halfway between `man tool` and `tool --help`
from machineconfig.utils.utils import get_latest_release, find_move_delete_linux
import crocodile.toolbox as tb
from platform import system


url = r'https://github.com/dbrgn/tealdeer'


def main(version=None):
    if system() == 'Windows':
        f = get_latest_release(url, version=version).joinpath('tealdeer-windows-x86_64-msvc.exe').download().rename('tldr.exe')
        f.move(folder=f.get_env().WindowsApps, overwrite=True)
    elif system() == 'Linux':
        f = get_latest_release(url, version=version).joinpath('tealdeer-linux-x86_64-musl').download().rename('tldr')
        find_move_delete_linux(f, 'tldr', delete=True)
        tb.Terminal().run("tldr --update")
    else:
        raise NotImplementedError(f"System {system()} not supported")


if __name__ == '__main__':
    main()




from platform import system
# import subprocess
import crocodile.toolbox as tb
from machineconfig.utils.utils import LIBRARY_ROOT, display_options


def main():
    direction = display_options(msg="BACKUP OR RETRIEVE?", options=["BACKUP", "RETRIEVE"], default="BACKUP")

    remotes = tb.L(tb.Terminal().run("rclone listremotes", shell="pwsh").op_if_successfull_or_default("").splitlines()).apply(lambda x: x.replace(":", ""))
    if len(remotes) == 0:
        raise RuntimeError(f"You don't have remotes. Configure your rclone first to get cloud services access.")
    cloud = display_options(msg="WHICH CLOUD?", options=list(remotes), default=remotes[0])
    bu_file = LIBRARY_ROOT.joinpath("profile/backup.toml").readit()

    if system() == "Linux": bu_file = {key: val for key, val in bu_file.items() if "windows" not in key}
    elif system() == "Windows": bu_file = {key: val for key, val in bu_file.items() if "linux" not in key}

    choice_key = display_options(msg="WHICH FILE of the following do you want to back up?", options=['all'] + list(bu_file.keys()), default="dotfiles")

    if choice_key == "all": items = bu_file
    else: items = {choice_key: bu_file[choice_key]}

    for item_name, item in items.items():
        if system().lower() in item_name: generic_os = False
        else: generic_os = True

        tb.S(item).print(as_config=True, title=direction)
        path = tb.P(item['path'])
        if direction == "BACKUP":
            path.to_cloud(cloud=cloud, zip=item['zip'], rel2home=True, encrypt=item['encrypt'], key=None, pwd=None, generic_os=generic_os)
        elif direction == "RETRIEVE":
            if item['zip']: path += ".zip"
            if item['encrypt']: path += ".enc"
            path.from_cloud(cloud=cloud, unzip=item['zip'], rel2home=True, decrypt=item['encrypt'], localpath=None, key=None, pwd=None, generic_os=generic_os)
        else: raise ValueError

    program = f"echo 'Finished backup.'"

    return program


if __name__ == "__main__":
    pass




from machineconfig.utils.utils import get_latest_release, find_move_delete_linux
from platform import system


url = 'https://github.com/tbillington/kondo'


def main(version=None):
    if system() == 'Windows':
        get_latest_release(url, download_n_extract=True, file_name='kondo-x86_64-pc-windows-msvc.zip', version=version)
    elif system() == 'Linux':
        downloaded = get_latest_release(url, download_n_extract=False, version=version).joinpath("kondo-x86_64-unknown-linux-gnu.tar.gz").download().ungz_untar(inplace=True)
        find_move_delete_linux(downloaded=downloaded, tool_name="kondo")
    else:
        raise NotImplementedError(f"System {system()} not supported")


if __name__ == '__main__':
    main()




"""
Rust version of FZF (GO)
"""

from machineconfig.utils.utils import get_latest_release
import crocodile.toolbox as tb

def main(version=None):
    _ = version
    # repo_url = tb.P(r"https://github.com/lotabout/skim")
    # release = get_latest_release(repo_url.as_url_str(), suffix='x86_64-unknown-linux-musl', compression='tar.gz', linux=True, strip_v=True, download_n_extract=True)
    pass


if __name__ == '__main__':
    main()




import crocodile.toolbox as tb
import argparse
# import subprocess


def args_parser():
    parser = argparse.ArgumentParser(description='Cloud Management CLI.')

    # positional argument
    parser.add_argument("cloud", help="rclone cloud profile name.")
    parser.add_argument("file", help="file/folder path.")
    # FLAGS
    parser.add_argument("--zip", "-z", help="Zip before sending.", action="store_true")  # default is False
    parser.add_argument("--encrypt", "-e", help="Encrypt before sending.", action="store_true")  # default is False
    parser.add_argument("--relative_to_home", "-r", help="set remote path as home relative local path.", action="store_true")  # default is False
    parser.add_argument("--share", "-s", help="Share file.", action="store_true")
    # optional argument
    parser.add_argument("--remote_dir", "-d", help="Remote directory to send to.", default="")
    parser.add_argument("--key", "-k", help="Key for encryption", default=None)
    parser.add_argument("--pwd", "-p", help="Password for encryption", default=None)

    args = parser.parse_args()
    tb.P(args.file).to_cloud(cloud=args.cloud, zip=args.zip_first, rel2home=args.relative_to_home,
                             share=args.share, key=args.key, pwd=args.pwd, encrypt=args.encrypt_first)


if __name__ == "__main__":
    args_parser()




from platform import system
from machineconfig.utils.utils import get_latest_release, tb


def main(version=None):
    url = get_latest_release('https://github.com/XAMPPRocky/tokei', download_n_extract=False, version=version)

    if system() == 'Windows':
        url.joinpath('tokei-x86_64-pc-windows-msvc.exe').download().move(folder=tb.get_env().WindowsApps, name='tokei.exe', overwrite=True)
    else:
        link = url.joinpath('tokei-x86_64-unknown-linux-gnu.tar.gz').download().ungz_untar(inplace=True)
        exe = link.joinpath('tokei')
        exe.chmod(0o777)
        # exe.move(folder=r"/usr/local/bin", overwrite=False)
        tb.Terminal().run(f"sudo mv {exe} /usr/local/bin/").print_if_unsuccessful(desc="MOVING executable to /usr/local/bin", strict_err=True, strict_returncode=True)


if __name__ == '__main__':
    main()





from machineconfig.utils.utils import get_latest_release
from platform import system
import crocodile.toolbox as tb

repo_url = tb.P(r"https://github.com/mgunyho/tere")


def main(version=None):
    if system() == 'Windows':
        from crocodile.environment import AppData
        target = AppData
        suffix = "x86_64-pc-windows-gnu"
        exe = get_latest_release(repo_url.as_url_str(), suffix=suffix, download_n_extract=True, delete=False, strip_v=True, version=version)
    else:
        release = get_latest_release(repo_url.as_url_str(), version=version)
        path = release.joinpath(f"tere-{str(release[-1]).replace('v', '')}-x86_64-unknown-linux-gnu.zip").download()
        exe = path.unzip(inplace=True).joinpath("tere")
        exe.chmod(0o777)
        # exe.move(folder=r"/usr/local/bin", overwrite=False)
        tb.Terminal().run(f"sudo mv {exe} /usr/local/bin/").print_if_unsuccessful(desc="MOVING executable to /usr/local/bin", strict_err=True, strict_returncode=True)


if __name__ == '__main__':
    main()




from machineconfig.utils.utils import get_latest_release, find_move_delete_linux
import crocodile.toolbox as tb


def main(version=None):
    _ = version
    # repo_url = tb.P(r"https://github.com/jarun/nnn")
    # release = get_latest_release(repo_url.as_url_str())
    # downloaded = release.joinpath(f"nnn-{release.name}.tar.gz").download().ungz_untar(inplace=True)
    # find_move_delete_linux(downloaded=downloaded, tool_name="nnn")
    #
    return ""


if __name__ == '__main__':
    main()




from machineconfig.utils.utils import get_latest_release
from platform import system
import crocodile.toolbox as tb

"""
VSCode and Pycharm have their own extensions from marketplace, but for other editors, we need to install this LSP manually.
"""

repo_url = tb.P(r"https://github.com/rust-lang/rust-analyzer")


def main(version=None):
    url = get_latest_release(repo_url.as_url_str(), download_n_extract=False, version=version)
    if system() == "Windows":
        url.joinpath(f"rust-analyzer-x86_64-pc-windows-msvc.gz").download().ungz(inplace=True).with_name("rust-analyzer.exe", inplace=True).move(folder="~/.cargo/bin", overwrite=True)
    else:
        url.joinpath(f"rust-analyzer-x86_64-unknown-linux-gnu.gz").download().ungz(inplace=True).with_name("rust-analyzer", inplace=True).move(folder="~/.cargo/bin", overwrite=True)
        # downloaded.move(folder=r"/usr/local/bin", overwrite=False)


if __name__ == '__main__':
    pass





from machineconfig.utils.utils import get_latest_release
import crocodile.toolbox as tb
import platform


url = "https://github.com/dalance/procs/"


def main(version=None):
    if platform.system() == "Windows":
        get_latest_release(url, suffix="x86_64-windows", download_n_extract=True, version=version)
    elif platform.system() == "Linux":
        get_latest_release(url, suffix="x86_64-linux", download_n_extract=True, linux=True, compression="zip", version=version)
    else:
        raise NotImplementedError(f"Platform {platform.system()} not supported.")


if __name__ == '__main__':
    main()




from machineconfig.utils.utils import get_latest_release, find_move_delete_linux
import crocodile.toolbox as tb

url = r'https://github.com/ClementTsang/bottom'
fname = 'bottom_x86_64-unknown-linux-gnu.tar.gz'


def main(version=None):
    release = get_latest_release(url, version=version)
    downloaded = tb.P(release).joinpath(fname).download().ungz_untar(inplace=True)
    find_move_delete_linux(downloaded, 'btm', delete=True)


if __name__ == '__main__':
    main()




from crocodile.environment import OneDriveExe, myhome
from crocodile.file_management import P
import argparse
import os


def main(which, file, z, encrypt_first, key, pwd, overwrite):
    if (onedrive_settings_path := P.home().joinpath("dotfiles/settings/paths.toml")).exists():
        onedrive = onedrive_settings_path.readit()['onedrive']
        onedrive = P.home().joinpath(onedrive[onedrive[which]])
    else:
        onedrive = P.home().joinpath(os.environ["OneDrive"])

    file = process_sent_file(file=file, zip_first=z, encrypt_first=encrypt_first, key=key, pwd=pwd)
    remote_dir = onedrive.joinpath(myhome, file.rel2home().parent)
    path = file.copy(folder=remote_dir, overwrite=overwrite)
    if z or encrypt_first: P(file).delete(sure=True)
    OneDriveExe()  # push to OneDrive


def arg_parser():
    parser = argparse.ArgumentParser(description='OneDrive Backup')

    # positional argument
    parser.add_argument("file", help="file/folder path to be backed up", default="")
    # FLAGS
    parser.add_argument("--recursive", "-r", help="Send recursively.", action="store_true")  # default is False
    parser.add_argument("--zip_first", "-z", help="Zip before backing up.", action="store_true")  # default is False
    parser.add_argument("--encrypt_first", "-e", help="Encrypt before backing up.", action="store_true")  # default is False
    parser.add_argument("--overwrite", "-o", help="Overwrite existing file.", action="store_true")  # default is False

    # optional argument
    parser.add_argument("--which", "-w", help="Which onedrive to use? see: cat: ~/dotfiles/settings/paths.toml", default="default")
    parser.add_argument("--remote_dir", "-d", help="Remote directory to send to.", default="")
    parser.add_argument("--relative_to_home", "-R", help="Relative to `myhome` folder", action="store_true")  # default is False
    parser.add_argument("--key", "-k", help="Key for encryption", default=None)
    parser.add_argument("--pwd", "-p", help="Password for encryption", default=None)

    args = parser.parse_args()
    main(which=args.which, file=args.file, z=args.zip_first, encrypt_first=args.encrypt_first, pwd=args.pwd, key=args.key, overwrite=True)


if __name__ == "__main__":
    arg_parser()




import crocodile.toolbox as tb
import crocodile.environment as env
from uuid import uuid4


"""
Not to be confused:
* Windows Terminal & Windows Terminal Preview: The latter is the night release version of WT.
* Windows PowerShell & PowerShell: The latter is community developed and is available on all platforms.
* Windows Powershell comes preinstalled with the system, Powershell must be installed separately.
* Lastly, powershell has a preview version as well.
All settings are available on GitHub: https://aka.ms/terminal-profiles-schema
"""


assert env.system == 'Windows', 'This script is only for Windows.'


class TerminalSettings(object):
    def __init__(self):
        # Grabbing Terminal Settings file:
        self.path = env.LocalAppData.joinpath(r"Packages\Microsoft.WindowsTerminal_8wekyb3d8bbwe\LocalState\settings.json")
        self.path.copy(append=".orig_" + tb.randstr())
        self.dat = self.path.readit()
        self.profs = tb.L(self.dat["profiles"]["list"])

    def save_terminal_settings(self):
        self.dat["profiles"]["list"] = list(self.profs)
        self.dat.save_json(self.path, indent=5)

    # ========================= Terminal Settings =========================================
    def update_default_settings(self):
        # Changing start up settings:
        self.dat["startOnUserLogin"] = True
        self.dat["launchMode"] = "fullscreen"
        self.dat["theme"] = "dark"
        self.dat["focusFollowMouse"] = True
        self.dat["copyOnSelect"] = True
        self.dat["profiles"]["defaults"]["padding"] = "0"
        self.dat["profiles"]["defaults"]["useAcrylic"] = False

    # 1- Customizing Powershell========================================================
    # as opposed to Windows Powershell
    def customize_powershell(self, nerd_font=True):
        pwsh = dict(name="PowerShell",
                    commandline="pwsh",
                    hidden=False,
                    opacity=87,
                    # guid="{" + str(uuid4()) + "}",  # WT doesn't accept any GUID to identify pwsh
                    startingDirectory="%USERPROFILE%",  # "%USERPROFILE%",   # None: inherent from parent process.
                    )
        if nerd_font: pwsh["font"] = dict(face="CaskaydiaCove Nerd Font")  # because oh-my-posh uses glyphs from this font.
        for idx, item in enumerate(self.profs):
            if item["name"] == "PowerShell":
                self.profs[idx].update(pwsh)
                break
        else: print(f"Couldn't customize powershell because profile not found, try to install it first.")

    def make_powershell_default_profile(self):
        for profile in self.profs:
            if profile["name"] == "PowerShell":
                self.dat["defaultProfile"] = profile["guid"]
                print("Powershell is now the default profile.")
                break
        else: print("Powershell profile was not found in the list of profile and therefore was not made the deafult.")

    def add_croshell(self):
        # Adding croshell if it is not there.
        # py_pr = tb.copy.deepcopy(pr)  # use it as a template for the new profile.
        import machineconfig
        lib_root = tb.P(machineconfig.__file__).parent.collapseuser().as_posix()
        croshell = dict(name="croshell",
                        guid="{" + str(uuid4()) + "}",
                        # commandline=f"powershell.exe -Command \"{activate} ipython -i -c 'from crocodile.toolbox import *'\"",
                        commandline=f'powershell.exe -Command "{lib_root}/scripts/windows/croshell.ps1"',
                        startingDirectory="%USERPROFILE%",  # "%USERPROFILE%",   # None: inherent from parent process.
                        )
        # startingDirectory = None means: inheret from parent process, which will is the default, which point to /System32
        # Launching a new profile with ctr+shift+2 is equivalent to: wt --profile croshell -d . --new-tab
        for profile in self.profs:
            if profile["name"] == "croshell":
                profile.update(croshell)
                break
        else: self.profs.append(croshell)

    def add_ubuntu(self):
        # Add Ubunto if it is not there.
        ubuntu = dict(name="Ubuntu",
                      commandline="wsl -d Ubuntu -- cd ~",
                      hidden=False,
                      guid="{" + str(uuid4()) + "}",
                      startingDirectory="%USERPROFILE%",  # "%USERPROFILE%",   # None: inherent from parent process.
                      )
        if self.profs.filter(lambda x: x["name"] == "Ubuntu").__len__() < 1: self.profs.append(ubuntu)

    def standardize_profiles_order(self):
        # Changing order of profiles:
        others = []
        pwsh = croshell = ubuntu = wpwsh = cmd = azure = None
        for profile in self.profs:
            name = profile["name"]
            if name == "PowerShell": pwsh = profile
            elif name == "croshell": croshell = profile
            elif name == "Ubuntu": ubuntu = profile
            elif name == "Windows PowerShell": wpwsh = profile
            elif name == "Command Prompt": cmd = profile
            elif name == "Azure Cloud Shell": azure = profile
            else: others.append(profile)
        self.profs = [item for item in [pwsh, croshell, ubuntu, wpwsh, cmd, azure] + others if item is not None]


def main():
    shell = {"powershell": "pwsh.exe", "Windows Powershell": "powershell.exe"}["powershell"].split(".exe")[0]
    if shell == "pwsh":
        ts = TerminalSettings()
        ts.update_default_settings()
        ts.customize_powershell(nerd_font=True)

        ts.make_powershell_default_profile()
        ts.add_croshell()
        ts.add_ubuntu()
        ts.standardize_profiles_order()
        ts.dat['actions'].append({'command': 'togglePaneZoom', 'keys': 'ctrl+shift+z'})
        ts.save_terminal_settings()
    else: raise NotImplementedError


if __name__ == '__main__':
    pass




import crocodile.toolbox as tb
# fd (rust)
# cargo install fd-find
# search for files/folder names only. Result can be fed to fzf for further filtering. For that use: fd | fzf. one can also make fd the searcher in fzf

url = 'https://github.com/ascii-boxes/boxes'


def main(version=None):
    from machineconfig.utils.utils import get_latest_release, tmp_install_dir

    get_latest_release(url, suffix="intel-win32", download_n_extract=True, version=version, strip_v=True, delete=False)
    tmp_install_dir.search("boxes.cfg", r=True)[0].move(folder=tb.P.get_env().WindowsApps, overwrite=True)
    tmp_install_dir.search("*boxes*", files=False).apply(lambda x: x.delete(sure=True))


if __name__ == '__main__':
    main()




import crocodile.toolbox as tb
# from machineconfig.utils.utils import display_options

def main():
    target = tb.P(input("Symlink to which target? ")).expanduser().absolute()
    source = input(f"symlink from which source ? [default to: CWD/{target.name}] ") or tb.P.cwd().joinpath(target.name)
    if type(source) is str: source = tb.P(source).expanduser().absolute()
    # ve_path = display_options(msg="symlin link? ", options=tb.P.home().joinpath("ve").starget.symlink_toimport crocodile.toolbox as tb; tb.P(r'$pwd').joinpath('venv').symlink_to(r'$to'); tb.P('.gitignore').modify_text('venv', 'venv', replace_line=True)"(target.symlink_to(
    # tb.P('.gitignore').modify_text('venv', 'venv', replace_line=True)"

    source.symlink_to(target, overwrite=True)
    return "echo 'Finished creating symlink.'"


if __name__ == '__main__':
    pass





import crocodile.toolbox as tb
import time
import pandas as pd
import platform
from rich.console import Console
# from rich.progress import track
from machineconfig.utils.utils import LIBRARY_ROOT
from machineconfig.jobs.python.python_linux_installers_all import get_installed_cli_apps
from tqdm import tqdm


safe_apps_records = LIBRARY_ROOT.joinpath(f"profile/records/{platform.system().lower()}/safe_cli_apps.json")
safe_apps_url = LIBRARY_ROOT.joinpath(f"profile/records/{platform.system().lower()}/safe_cli_apps_url.txt")
cloud = "gdpo"


def scan(path, pct=0.0):
    vt = tb.install_n_import(package="vt", name="vt-py")
    client = vt.Client(tb.P.home().joinpath("dotfiles/creds/tokens/virustotal").read_text().split("\n")[0])
    console = Console()
    console.rule(f"Scanning {path}. {pct:.2f}% done")
    if path.is_dir():
        print(f"Skipping {path} as it is a directory.")
        return None
    with open(str(path), "rb") as f:
        analysis = client.scan_file(f)

    repeat_counter = 0
    while True:
        with console.status(f"waiting for scan of {path} ... "):
            try:
                anal = client.get_object("/analyses/{}", analysis.id)
                # print(anal.status)
                if anal.status == "completed": break
            except:
                repeat_counter += 1
                if repeat_counter > 2:
                    raise ValueError(f"Error in scanning {path}")
                print(f"Error in scanning, trying again.")
            time.sleep(30)

    df = pd.DataFrame(anal.results).T
    malicious = []
    for idx, row in df.iterrows():
        if row.result is None and row.category in ["undetected", "type-unsupported", "failure", "timeout", "confirmed-timeout"]: continue
        else:
            tb.Struct(row.to_dict()).print(as_config=True, title=f"Found Category {row.category}")
            malicious.append(row)

    positive_pct = len(malicious) / len(df) * 100
    print(f"positive_ratio = {positive_pct:.1f} %")
    return positive_pct


def main():
    apps_paths_raw = get_installed_cli_apps()

    app_versions = []
    versions = tb.P.home().joinpath(f"tmp_results/cli_tools_installers/versions").search()
    for an_app in apps_paths_raw:
        if an_app.stem in versions.stem: app_versions.append(versions.filter(lambda x: x.stem == an_app.stem.replace(".exe", ""))[0].read_text())
        else: app_versions.append(None)

    positive_pct = []
    # for app in track(apps, description="App apps scanning..."):
    for idx, app in enumerate(apps_paths_raw):
        try: res = scan(app, idx/len(apps_paths_raw)*100)
        except ValueError as ve:
            print(ve)
            res = None
        positive_pct.append(res)

    res_df = pd.DataFrame({"app_name": apps_paths_raw.stem.list, "version": app_versions, "positive_pct": positive_pct,
                           "app_path": apps_paths_raw.apply(lambda x: x.collapseuser(strict=False).as_posix()).list})

    apps_safe_df = res_df.query("positive_pct < 5")

    app_url = []
    for idx, row in tqdm(apps_safe_df.iterrows(), total=apps_safe_df.shape[0]):
        apps_safe_url = upload(tb.P(row["app_path"]).expanduser())
        app_url.append(apps_safe_url.as_posix() if type(apps_safe_url) == tb.P else apps_safe_url)
    res_df["app_url"] = app_url

    res_df.to_json(safe_apps_records.create(parents_only=True), index=False, orient='split')
    share_url = safe_apps_records.to_cloud(cloud, rel2home=True, share=True)
    safe_apps_url.write_text(share_url.as_posix())
    print(res_df)


def upload(path):
    tb.install_n_import("call_function_with_timeout")
    from call_function_with_timeout import SetTimeout
    def func():
        return path.to_cloud(cloud, rel2home=True, share=True)
    func_with_timeout = SetTimeout(func, timeout=180)
    is_done, is_timeout, erro_message, results = func_with_timeout()
    if is_done: return results
    else: return None


def download_rclone_links(url, name=None):
    if "drive.google.com" in str(url):
        url = str(url).replace("open?", "uc?")
    else: raise NotImplementedError("Only google drive is supported for now.")
    return tb.P(url).download(name=name)


def read_safe_apps_records(from_cloud=True):
    if from_cloud:
        file = download_rclone_links(safe_apps_url.read_text(), name=safe_apps_records.name)
    else:
        file = safe_apps_records
    tmp = file.readit()
    df = pd.DataFrame(tmp['data'], columns=tmp['columns'])
    return df


def download_safe_apps():
    df = read_safe_apps_records(from_cloud=True)

    if platform.system().lower() == "windows":
        from machineconfig.jobs.python.python_windows_installers_all import get_cli_py_installers
        cli_installers = get_cli_py_installers()
    elif platform.system().lower() == "linux":
        from machineconfig.jobs.python.python_linux_installers_all import get_cli_py_installers
        cli_installers = get_cli_py_installers()
    else: raise NotImplementedError(f"Platform {platform.system().lower()} is not supported yet.")

    def install_cli_apps(row):
        if row["app_url"] is None:
            print(f"{row['app_name']} url is not available, trying to download from its sources:")
            version = row["version"]
            if version is None:
                print(f"Version is not available, either, I can't trust the latest version. Skipping installation of this program.")
                return False
            else:
                tmp = cli_installers.filter(lambda x: x.stem == row["app_name"].replace(".exe", ""))
                if len(tmp) == 0:
                    print(f"Can't find the installer for {row['app_name']}, skipping installation of this program.")
                    return False
                else: tb.Read.py(tmp[0])["main"](version=version)
        else:
            name = row["app_name"]
            if platform.system().lower() == "windows" and not name.endswith(".exe"): name += ".exe"
            exe = download_rclone_links(row["app_url"], name=name)
            if platform.system().lower() == "linux":
                tb.Terminal().run(f"chmod +x {exe}")
                tb.Terminal().run(f"mv {exe} /usr/local/bin/")
            elif platform.system().lower() == "windows":
                exe.move(folder=tb.P.home().joinpath("AppData/Local/Microsoft/WindowsApps"), overwrite=True)
        return True

    res = tb.L(df.iterrows()).apply(lambda x: install_cli_apps(x[1]), jobs=20)

    print("\n"*3)
    for item_flag, item_name in zip(res, df["app_name"]):
        if item_flag: print(f"✅ {item_name} is installed. 😁")
        else: print(f"❌ {item_name} failed to install for reasons explained in the log above, try manually.")


if __name__ == '__main__':
    pass




import crocodile.toolbox as tb
import machineconfig.jobs.python_linux_installers as inst
import machineconfig.jobs.python_generic_installers as gens
import platform


def get_cli_py_installers():
    return tb.P(inst.__file__).parent.search("*.py", filters=[lambda x: "__init__" not in str(x)]) + tb.P(gens.__file__).parent.search("*.py", filters=[lambda x: "__init__" not in str(x)])


def get_installed_cli_apps():
    if platform.system() == "Windows": apps = tb.P.home().joinpath("AppData/Local/Microsoft/WindowsApps").search("*.exe", not_in=["notepad"])
    elif platform.system() == "Linux": apps = tb.P(r"/usr/local/bin").search("*")
    else: raise NotImplementedError("Not implemented for this OS")
    apps = tb.L([app for app in apps if app.size("kb") > 0.1 and not app.is_symlink()])  # no symlinks like paint and wsl and bash
    return apps


def install_logic(py_file, version=None):
    try:
        old_version = tb.Terminal().run(f"{py_file.stem} --version", shell="powershell").op.replace("\n", "")
        tb.Read.py(py_file)["main"](version=version)
        new_version = tb.Terminal().run(f"{py_file.stem} --version", shell="powershell").op.replace("\n", "")
        if old_version == new_version: return f"😑 {py_file.stem}, same version: {old_version}"
        else: return f"🤩 {py_file.stem} updated from {old_version} === to ===> {new_version}"
    except Exception as ex:
        print(ex)
        return f"Failed at {py_file.stem} with {ex}"


def main(installers=None, safe=False):
    if safe:
        from machineconfig.jobs.python.check_installations import safe_apps_url
        apps_dir = tb.P(safe_apps_url.read_text()).download().unzip(inplace=True)
        if platform.system().lower() == "windows":
            apps_dir.search("*").apply(lambda app: app.move(folder=tb.P.get_env().WindowsApps))
        elif platform.system().lower() == "linux":
            tb.Terminal().run(f"sudo mv {apps_dir.as_posix()}/* /usr/local/bin/").print_if_unsuccessful(desc="MOVING executable to /usr/local/bin", strict_err=True, strict_returncode=True)
        else: raise NotImplementedError(f"I don't know this system {platform.system()}")
        apps_dir.delete(sure=True)
        return None

    installers = installers if installers is not None else tb.L(get_cli_py_installers())

    install_logic(installers[0])  # try out the first installer alone cause it will ask for password, so the rest will inherit the sudo session.

    # summarize results
    res = installers[1:].apply(install_logic, jobs=10)
    from rich.console import Console
    console = Console()
    print("\n")
    console.rule("Same version apps")
    print(f"{res.filter(lambda x: 'same version' in x).print()}")
    print("\n")
    console.rule("Updated apps")
    print(f"{res.filter(lambda x: 'updated from' in x).print()}")
    print("\n")
    console.rule("Failed apps")
    print(f"{res.filter(lambda x: 'Failed at' in x).print()}")

    print("\n")
    print("Completed Installation".center(100, "-"))
    print("\n" * 2)


if __name__ == '__main__':
    main()





from machineconfig.utils.utils import get_latest_release
from platform import system


url = "https://github.com/Byron/dua-cli"


def main(version=None):
    if system() == 'Windows':
        suffix = "x86_64-pc-windows-msvc"
        _ = get_latest_release(url, tool_name="dua", suffix=suffix, download_n_extract=True, delete=False, strip_v=False, compression="zip", version=version)
    else:
        suffix = "x86_64-unknown-linux-musl"
        _ = get_latest_release(url, tool_name="dua", download_n_extract=True, delete=True, suffix=suffix, compression="tar.gz", linux=True, version=version)
    return ""


if __name__ == '__main__':
    main()





import crocodile.toolbox as tb
# from platform import system
from machineconfig.utils.utils import display_options


def main():
    private_keys = tb.P.home().joinpath(".ssh").search("*.pub").apply(lambda x: x.with_name(x.stem)).filter(lambda x: x.exists())
    choice = display_options(msg="Path to private key to be used when ssh'ing: ", options=private_keys.list + ["I have the path to the key file", "I want to paste the key itself"])
    if choice == "I have the path to the key file": path_to_key = tb.P(input("Input path here: ")).expanduser().absolute()
    elif choice == "I want to paste the key itself": path_to_key = tb.P.home().joinpath(f".ssh/{input('file name (default: my_pasted_key): ') or 'my_pasted_key'}").write_text(input("Paste the private key here: "))
    else: path_to_key = tb.P(choice)
    txt = f"IdentityFile {path_to_key.collapseuser().as_posix()}"  # adds this id for all connections, no host specified.
    config_path = tb.P.home().joinpath(".ssh/config")
    if config_path.exists(): config_path.modify_text(txt_search=txt, txt_alt=txt, replace_line=True, notfound_append=True, prepend=True)  # note that Identity line must come on top of config file otherwise it won't work, hence `prepend=True`
    else: config_path.write_text(txt)
    program = f"echo 'Finished adding identity to ssh config file. {'*'*50} Consider reloading config file.'"
    return program


if __name__ == '__main__':
    pass




import crocodile.toolbox as tb
import crocodile.environment as env
from machineconfig.utils.utils import get_latest_release

"""
setup file for each shell can be found in $profile. The settings.json is the config file for Terminal.
"""


def install_nerd_fonts():
    # Step 1: download the required fonts that has all the glyphs and install them.
    folder = get_latest_release("https://github.com/ryanoasis/nerd-fonts").joinpath("CascadiaCode.zip").download().unzip(inplace=True)
    folder.search("*Windows*").delete(sure=True)
    folder.search("*readme*").delete(sure=True)
    folder.search("*LICENSE*").delete(sure=True)
    file = tb.P.tmpfile(suffix=".ps1").write_text(tb.P(__file__).with_name("install_fonts.ps1").read_text().replace(r".\fonts-to-be-installed", str(folder)))
    tb.subprocess.run(rf"powershell.exe -executionpolicy Bypass -nologo -noninteractive -File {file.str}")


def change_shell_profile():
    # Customize powershell profile such that it loads oh-my-posh and the terminal icons automatically.
    # Add arrow keys history functionality to the terminal.

    shell = {"powershell": "pwsh.exe", "Windows Powershell": "powershell.exe"}["powershell"].split(".exe")[0]
    profile_path = tb.Terminal().run("$profile", shell=shell).op2path()
    theme_path = env.LocalAppData.joinpath(r"Programs\oh-my-posh\themes").collapseuser().as_posix().replace("~", "$env:USERPROFILE")  # organization machine with homeshare confuse H: with ~.
    txt = f"""
oh-my-posh --init --shell pwsh --config {theme_path}\\jandedobbeleer.omp.json | Invoke-Expression
Import-Module -Name Terminal-Icons

# Shows navigable menu of all options when hitting Tab
Set-PSReadlineKeyHandler -Key Tab -Function MenuComplete
Set-PSReadlineKeyHandler -Key UpArrow -Function HistorySearchBackward
Set-PSReadlineKeyHandler -Key DownArrow -Function HistorySearchForward
Set-PSReadlineOption -PredictionViewStyle History
# see dynamic help with prerelease.
"""
    profile_path.modify_text(txt_search=txt, txt_alt=txt, replace_line=True, notfound_append=True)


def main():
    install_nerd_fonts()
    change_shell_profile()


if __name__ == '__main__':
    pass




# fzf (glang)
# search for files/folder names, but can come with preview option. For that use: my variant fzz.ps1
from machineconfig.utils.utils import get_latest_release
import crocodile.toolbox as tb


url = 'https://github.com/junegunn/fzf'

def main(version=None):
    # tb.Terminal().run("nu -c 'ps | where name == fzf.exe | each { |it| kill $it.pid --force}'", shell="pwsh").print()
    tb.L(tb.install_n_import("psutil").process_iter()).filter(lambda x: x.name() == 'fzf.exe').apply(lambda x: x.kill())
    get_latest_release(url, suffix='windows_amd64', download_n_extract=True, version=version)


if __name__ == '__main__':
    pass





"""
Rust version of lf (GO)
"""

from machineconfig.utils.utils import get_latest_release, find_move_delete_linux
import crocodile.toolbox as tb

# repo_url = tb.P(r"https://github.com/kamiyaa/joshuto")
# release = get_latest_release(repo_url.as_url_str())
# path = release.joinpath(f"joshuto-{release[-1]}-x86_64-unknown-linux-gnu.tar.gz")

def main(version=None):
    _ = version
    # pre-release
    path = tb.P(r'https://github.com/kamiyaa/joshuto/releases/download/v0.9.4/joshuto-v0.9.4-x86_64-unknown-linux-gnu.tar.gz')
    path = path.download().ungz_untar(inplace=True)
    exe = path.search()[0].joinpath("joshuto")
    exe.chmod(0o777)
    tb.Terminal().run(f"sudo mv {exe} /usr/local/bin/").print()
    path.delete(sure=True)
    # after first release:
    # url = get_latest_release("https://github.com/kamiyaa/joshuto", suffix="x86_64-unknown-linux-gnu", compression="tar.gz", download_n_extract=True)


if __name__ == '__main__':
    main()



import crocodile.toolbox as tb
from machineconfig.utils.utils import display_options


descriptive_themes = ["cupcake", "doubletime", "zork", "slick", "powerline-multiline", "font"]


def main(new_theme=None):
    profile = tb.P.home().joinpath(".bashrc")
    current_theme = tb.L(profile.read_text().splitlines()).filter(lambda x: "OSH_THEME=" in x)[0].split("=")[1]
    themes = tb.P.home().joinpath(".oh-my-bash/themes").search("*", not_in=["THEMES.md"]).apply(lambda x: x.trunk)
    themes.list.sort()
    if new_theme is None:
        tail = "Recomm: "
        new_theme = display_options(msg=f"Choose a theme number from the list above: ", options=list(themes) + ["slick", "surprise me"], default="surprise me", tail=tail,
                                    prompt=f"Recommended descriptive ones are {descriptive_themes}")
    if new_theme == "surprise me": new_theme = themes.sample()[0]
    print("Current Theme:", current_theme)
    print("New theme: ", f'"{new_theme}"')  #
    profile.modify_text(txt_search=current_theme, txt_alt=f'"{new_theme}"', replace_line=False)


if __name__ == '__main__':
    pass




import crocodile.toolbox as tb
from platform import system
from machineconfig.utils.utils import LIBRARY_ROOT, display_options


def get_add_ssh_key_script(path_to_key):
    if system() == "Linux": authorized_keys = tb.P.home().joinpath(".ssh/authorized_keys")
    elif system() == "Windows": authorized_keys = tb.P("C:/ProgramData/ssh/administrators_authorized_keys")
    else: raise NotImplementedError

    if authorized_keys.exists():
        split = "\n"
        print(f'Users that can access this machine have private keys of those pub keys:\n{authorized_keys.read_text().split(split)}')

        if path_to_key.read_text() in authorized_keys.read_text():
            program = ""
        else:
            if system() == "Linux":
                program = f"cat {path_to_key} >> ~/.ssh/authorized_keys"
            else:
                program = LIBRARY_ROOT.joinpath("jobs/windows/openssh-server_add_key.ps1")
                program = tb.P(program).expanduser().read_text().replace('$sshfile=""', f'$sshfile="{path_to_key}"')

    else:
        if system() == "Linux":
            program = f"cat {path_to_key} > ~/.ssh/authorized_keys"
        else:
            program = LIBRARY_ROOT.joinpath("jobs/windows/openssh-server_add_key.ps1")
            program = tb.P(program).expanduser().read_text().replace('$sshfile=""', f'$sshfile="{path_to_key}"')
    return program


def main():
    pub_keys = tb.P.home().joinpath(".ssh").search("*.pub")
    res = display_options("Which public key to add? ", options=pub_keys.list + [f"all pub keys available ({len(pub_keys)})", "I have the path to the key file", "I want to paste the key itself"])
    if res == "all": program = "\n\n\n".join(pub_keys.apply(get_add_ssh_key_script))
    elif res == "I have the path to the key file": program = get_add_ssh_key_script(tb.P(input("Path: ")).expanduser().absolute())
    elif res == "I want to paste the key itself": program = get_add_ssh_key_script(tb.P.home().joinpath(f".ssh/{input('file name (default: my_pasted_key.pub): ') or 'my_pasted_key.pub'}").write_text(input("Paste the pub key here: ")))
    else: program = get_add_ssh_key_script(tb.P(res))
    return program


if __name__ == '__main__':
    pass




"""
This script Takes away all config files from the computer, place them in one directory
`dotfiles`, and create symlinks to those files from thier original locations.
"""

import crocodile.toolbox as tb
from crocodile.environment import DotFiles, system, PathVar, UserName  # ProgramFiles, WindowsApps  # , exe
from machineconfig.utils.utils import symlink, LIBRARY_ROOT, REPO_ROOT, display_options
# import os
import subprocess
from rich.console import Console


ERROR_LIST = []  # append to this after every exception captured.
CONFIG_ROOT = LIBRARY_ROOT.parent.parent.joinpath("settings")
OTHER_SYSTEM = "windows" if system == "Linux" else "linux"

# =================== SYMLINKS ====================================


def link_ssh(overwrite=True):
    """The function can link aribtrary number of files without linking the directory itself (which is not doable in toml config file)"""
    path = tb.P.home().joinpath(".ssh")
    target = DotFiles.joinpath("creds/.ssh")
    for item in target.search("*"):
        # if "authorized_keys" in item: continue
        symlink(path.joinpath(item.name), item, overwrite=overwrite)
    if system == "Linux":  # permissions of ~/dotfiles/.ssh should be adjusted
        try:
            subprocess.run(f"chmod 700 ~/.ssh/")
            subprocess.run(f"chmod 700 {target.collapseuser().as_posix()}/")  # may require sudo
            subprocess.run(f"chmod 600 {target.collapseuser().as_posix()}/*")
        except Exception as e:
            ERROR_LIST.append(e)
            print("Caught error", e)


def link_aws(overwrite=True):
    path = tb.P.home().joinpath(".aws")
    target = DotFiles.joinpath("aws/.aws")
    for item in target.search("*"): symlink(path.joinpath(item.name), item, overwrite=overwrite)


def main_symlinks(choice=None):
    symlink_mapper = LIBRARY_ROOT.joinpath("profile/mapper.toml").readit()
    symlink_mapper['wsl_windows']['home']["to_this"] = symlink_mapper['wsl_windows']['home']["to_this"].replace("username", UserName)
    symlink_mapper['wsl_linux']['home']["to_this"] = symlink_mapper['wsl_linux']['home']["to_this"].replace("username", UserName)

    overwrite = True
    exclude = ["autostart_windows"]  # "wsl_linux", "wsl_windows"

    program_keys_raw = list(symlink_mapper.keys()) + ["aws", "ssh"]
    program_keys = []
    for program_key in program_keys_raw:
        if program_key in exclude or OTHER_SYSTEM in program_key:
            # print(f"Skipping {program_key} for {system}")
            continue
        else: program_keys.append(program_key)

    program_keys.sort()
    if choice is None:
        choice = display_options(msg="Which symlink to create?", options=program_keys + ["all", "none"], default="none")
        if str(choice) == "none": return

        overwrite = display_options(msg="Overwrite existing source file?", options=["yes", "no"], default="yes") == "yes"

    if str(choice) == "all" and system == "Windows" and not tb.Terminal.is_user_admin():
        print("*" * 200)
        raise RuntimeError(f"Run terminal as admin and try again, otherwise, there will be too many popups for admin requests and no chance to terminate the program.")
    elif choice == "all": program_keys = program_keys
    else: program_keys = [choice]

    for program_key in program_keys:
        if program_key == "aws":
            link_aws(overwrite=overwrite)
            continue
        elif program_key == "ssh":
            link_ssh(overwrite=overwrite)
            continue
        for file_key, file_map in symlink_mapper[program_key].items():
            try: symlink(this=file_map['this'], to_this=file_map['to_this'].replace("REPO_ROOT", REPO_ROOT.as_posix()).replace("LIBRARY_ROOT", LIBRARY_ROOT.as_posix()), overwrite=overwrite)
            except Exception as ex: print("Config error: ", program_key, file_key, "missing keys 'this ==> to_this'.", ex)

    if system == "Linux": tb.Terminal().run(f'chmod +x {LIBRARY_ROOT.joinpath(f"scripts/{system.lower()}")} -R')


# --------------------------------------- SHELL PROFILE --------------------------------------------------------
# The following is not a symlink creation, but modification of shell profile by additing dirs to PATH
# Shell profile is either in dotfiles and is synced (as in Windows), hence no need for update, or is updated on the fly (for Linux)
# for windows it won't change the profile, if the profile was modified already e.g. due to syncing


def get_shell_profile_path():
    if system == "Windows": profile_path = tb.Terminal().run("$profile", shell="pwsh").op2path()
    elif system == "Linux": profile_path = tb.P("~/.bashrc").expanduser()
    else: raise ValueError(f"Not implemented for this system {system}")
    print(f"Working on shell profile `{profile_path}`")
    return profile_path


def main_env_path(choice=None, profile_path=None):
    env_path = LIBRARY_ROOT.joinpath("profile/env_path.toml").readit()
    dirs = env_path[f'path_{system.lower()}']['extension']

    print(f"Current PATH: ", "\n============")
    tb.get_env().PATH.print()

    if choice is None:
        choice = display_options(msg="Which directory to add?", options=dirs + ["all", "none"], default="none")
        if str(choice) != "all": dirs = [choice]
    if choice == "none": return

    addition = PathVar.append_temporarily(dirs=dirs)
    profile_path = profile_path or get_shell_profile_path()
    profile_path.copy(name=profile_path.name + f".orig_" + tb.randstr())
    profile_path.modify_text(addition, addition, replace_line=False, notfound_append=True)


def main_add_sources_to_shell_profile(profile_path=None, choice=None):
    sources = LIBRARY_ROOT.joinpath("profile/sources.toml").readit()[system.lower()]['files']

    if choice is None:
        choice = display_options(msg="Which patch to add?", options=sources + ["all", "none"], default="none")
        if str(choice) != "all": sources = [choice]
    if choice == "none": return

    profile_path = profile_path or get_shell_profile_path()
    profile = profile_path.read_text()

    for a_file in sources:
        file = a_file.replace("REPO_ROOT", REPO_ROOT.as_posix()).replace("LIBRARY_ROOT", LIBRARY_ROOT.as_posix())
        file = tb.P(file).collapseuser().as_posix()  # this makes the shell profile interuseable across machines.
        if file not in profile:
            if system == "Windows": profile += f"\n. {file}"
            elif system == "Linux": profile += f"\nsource {file}"
            else: raise ValueError(f"Not implemented for this system {system}")
        else: print(f"SKIPPED source `{file}`, it is already sourced in shell profile.")
    profile_path.write_text(profile)


def main_add_patches_to_shell_profile(profile_path=None, choice=None):
    patches = LIBRARY_ROOT.joinpath(f"profile/patches/{system.lower()}").search()

    if choice is None:
        choice = display_options(msg="Which patch to add?", options=patches.list + ["all", "none"], default="none")
    if choice == "none": return
    if str(choice) == "all": patches = patches
    else: patches = [choice]

    profile_path = profile_path or get_shell_profile_path()
    profile = profile_path.read_text()

    for patch_path in patches:
        patch = patch_path.read_text()
        if patch in profile: print(f"Skipping `{patch_path.name}`; patch already in profile")
        else: profile += "\n" + patch

    if system == "Linux":
        res = tb.Terminal().run("cat /proc/version").op
        if "microsoft" in res.lower() or "wsl" in res.lower():
            profile += "\ncd ~"  # this is to make sure that the current dir is not in the windows file system, which is terribly slow and its a bad idea to be there anyway.

    profile_path.write_text(profile)


def main(choice=None):
    console = Console()
    print("\n")
    console.rule(f"CREATING SYMLINKS")

    # the only common choice among all programs below is "all".
    main_symlinks(choice=choice)
    print("\n")
    console.rule(f"ADDING ENV PATH")
    main_env_path(choice=choice)
    print("\n")
    console.rule(f"ADDING SOURCES TO SHELL PROFILE")
    main_add_sources_to_shell_profile(choice=choice)
    print("\n")
    console.rule(f"ADDING PATCHES TO SHELL PROFILE")
    main_add_patches_to_shell_profile(choice=choice)


if __name__ == '__main__':
    pass




# a TUI like winscp

from machineconfig.utils.utils import get_latest_release
from platform import system
import crocodile.toolbox as tb


repo_url = tb.P(r"https://github.com/veeso/termscp")


def main(version=None):
    if system() == 'Windows':
        suffix = "x86_64-pc-windows-msvc"
        exe = get_latest_release(repo_url.as_url_str(), suffix=suffix, download_n_extract=True, delete=False, version=version)
    else:
        suffix = "x86_64-unknown-linux-gnu"
        release = get_latest_release(repo_url.as_url_str(), download_n_extract=True, delete=True, suffix=suffix, compression="tar.gz", linux=True, version=version)
    return ""


if __name__ == '__main__':
    main()




from machineconfig.utils.utils import get_latest_release
from platform import system
# import crocodile.toolbox as tb


url = r"https://github.com/gopasspw/gopass"


def main(version=None):
    if system() == "Windows":
        get_latest_release(url, suffix="windows-amd64", download_n_extract=True, strip_v=True, tool_name="gopass", delete=False, version=version)
    elif system() == "Linux":
        get_latest_release(url, suffix="linux-amd64", download_n_extract=True, linux=True, compression="tar.gz", strip_v=True, tool_name="gopass", delete=False, version=version)
    else: raise NotImplementedError(f"System {system()} is not supported")


if __name__ == '__main__':
    main()





import argparse
import configparser
from pathlib import Path
# import random
# import string
import os


def createNewConnection(name, SSID, password):
    config = """<?xml version=\"1.0\"?>
<WLANProfile xmlns="http://www.microsoft.com/networking/WLAN/profile/v1">
    <name>"""+name+"""</name>
    <SSIDConfig>
        <SSID>
            <name>"""+SSID+"""</name>
        </SSID>
    </SSIDConfig>
    <connectionType>ESS</connectionType>
    <connectionMode>auto</connectionMode>
    <MSM>
        <security>
            <authEncryption>
                <authentication>WPA2PSK</authentication>
                <encryption>AES</encryption>
                <useOneX>false</useOneX>
            </authEncryption>
            <sharedKey>
                <keyType>passPhrase</keyType>
                <protected>false</protected>
                <keyMaterial>"""+password+"""</keyMaterial>
            </sharedKey>
        </security>
    </MSM>
</WLANProfile>"""
    command = "netsh wlan add profile filename=\""+name+".xml\""+" interface=Wi-Fi"
    with open(name+".xml", 'w') as file:
        file.write(config)
    os.system(command)


def connect(name, SSID):
    command = "netsh wlan connect name=\""+name+"\" ssid=\""+SSID+"\" interface=Wi-Fi"
    os.system(command)


def displayAvailableNetworks():
    command = "netsh wlan show networks interface=Wi-Fi"
    os.system(command)


def main():
    creds = configparser.ConfigParser()
    creds.read(Path.home().joinpath('dotfiles/creds/msc/wifi.ini'))

    parser = argparse.ArgumentParser(description='Wifi Connector')
    parser.add_argument('-n', "--ssid", help=f"SSID of Wifi", default='MyPhoneHotSpot')

    args = parser.parse_args()
    ssid = creds[args.ssid]['SSID']
    pwd = creds[args.ssid]['pwd']

    # displayAvailableNetworks()
    # createNewConnection(name, name, password)
    connect(ssid, ssid)


if __name__ == '__main__':
    main()





# as per https://github.com/Luzifer/ots
# this script is pure python and connects to server to create a secret
# or use ots executable to do this job


import requests
import crocodile.file_management as fm


from Crypto.Cipher import AES
from Crypto.Hash import MD5
import base64

def encrypt(key, pwd):
    pwd = pwd.encode("utf-8")
    hash_object = MD5.new(key.encode("utf-8"))
    key = hash_object.digest()
    iv = b'\x00' * 16
    cipher = AES.new(key, AES.MODE_CBC, iv)
    pad = 16 - (len(pwd) % 16)
    pwd += bytes([pad] * pad)
    encrypted_password = cipher.encrypt(pwd)
    return base64.b64encode(encrypted_password).decode("utf-8")


secret = input("Secret: ")
password = input("Password: ") or None


if password is not None: encoded_secret = encrypt(password, secret)
else: encoded_secret = secret


url = "https://ots.fyi/api/create"

payload = {"secret": encoded_secret}
headers = {'Content-Type': 'application/json'}

response = requests.post(url, json=payload, headers=headers)

if response.status_code == 201:
    res = response.json()
    print(res)
    assert res["success"] is True, "Request have failed"
    share_url = fm.P(f"https://ots.fyi/#{res['secret_id']}") + (f"|{password}" if password is not None else "")
    print(repr(share_url))
else:
    print("Request failed")
    raise RuntimeError(response.text)





"""
It has a funny installer that is a library of python scripts, not exe.
"""

from machineconfig.utils.utils import get_latest_release, find_move_delete_linux
import crocodile.toolbox as tb


def main(version=None):
    _ = get_latest_release(r"https://github.com/ranger/ranger", linux=True, download_n_extract=False, version=version)
    find_move_delete_linux(downloaded=tb.P("https://ranger.github.io/ranger-stable.tar.gz").download().ungz_untar(), tool_name="ranger")

    return ""


if __name__ == '__main__':
    main()





from machineconfig.utils.utils import get_latest_release, find_move_delete_windows, find_move_delete_linux
import crocodile.toolbox as tb
import platform


url = "https://github.com/sniptt-official/ots"


def main(version=None):
    if platform.system() == "Windows":
        latest = get_latest_release(url, download_n_extract=False, version=version)
        downloaded = latest.joinpath(f'ots_{latest[-1].str.replace("v", "")}_windows_amd64.zip').download()
        find_move_delete_windows(downloaded=downloaded.unzip(inplace=True), tool_name="ots",  delete=True)
    elif platform.system() == "Linux":
        latest = get_latest_release(url, suffix="x86_64-linux", download_n_extract=True, linux=True, compression="zip", version=version)
        downloaded = latest.joinpath(f'ots_{latest[-1].str.replace("v", "")}_linux_amd64.tar.gz').download()
        find_move_delete_linux(downloaded=downloaded.ungz_untar(inplace=True), tool_name="ots",  delete=True)
    else:
        raise NotImplementedError(f"Platform {platform.system()} not supported.")


if __name__ == '__main__':
    main()




import setuptools
from src.machineconfig import __version__

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="machineconfig",
    version=__version__,
    author="Alex Al-Saffar",
    author_email="programmer@usa.com",
    description="Dotfiles management package",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/thisismygitrepo/machineconfig",
    project_urls={
        "Bug Tracker": "https://github.com/thisismygitrepo/machineconfig/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.9",
)

# todo create a link to where this setup is. in .machoneconfig




# from crocodile.environment import OneDriveConsumer, OneDriveCommercial
from crocodile.environment import OneDriveExe
import crocodile.toolbox as tb
import argparse
from crocodile.comms.helper_funcs import process_retrieved_file
import os
import time


def main(file, unzip, decrypt, which, overwrite, key, pwd):
    if (onedrive_settings_path := tb.P.home().joinpath("dotfiles/settings/paths.toml")).exists():
        onedrive = onedrive_settings_path.readit()['onedrive']
        onedrive = tb.P.home().joinpath(onedrive[onedrive[which]])
    else:
        onedrive = tb.P.home().joinpath(os.environ["OneDrive"])

    target_file = tb.P(file).expanduser().absolute()
    source_file = onedrive.joinpath(tb.get_env().myhome, target_file.rel2home())
    if unzip and decrypt: source_file = source_file + ".zip.enc"

    print(f"Retriveing `{source_file}` \nTo `{target_file.parent}` folder.")
    if source_file.exists(): pass
    else:
        OneDriveExe()
        trials = 5
        while trials > 0 and not source_file.exists():
            trials -= 1
            print(f"Source file doesn't exist, waiting for OneDrive to sync ...")
            time.sleep(3)
    if not source_file.exists(): raise FileNotFoundError(f"Could not read the file on the could.")
            
    tmp_file = source_file.copy(folder=target_file.parent, overwrite=overwrite)  # make sure to avoid doing decryption in the storage site.
    process_retrieved_file(tmp_file, decrypt=decrypt, unzip=unzip, key=key, pwd=pwd)


def arg_parser():
    parser = argparse.ArgumentParser(description='OneDrive Backup')

    # positional argument
    parser.add_argument("file", help="file/folder path to be received.", default="")
    # FLAGS
    parser.add_argument("--recursive", "-r", help="Send recursively.", action="store_true")  # default is False
    parser.add_argument("--decrypt", "-e", help="Decrypt after receiving.", action="store_true")  # default is False
    parser.add_argument("--unzip", "-z", help="unzip after receiving.", action="store_true")  # default is False
    parser.add_argument("--overwrite", "-o", help="Overwrite existing file.", action="store_true")  # default is False
    # optional argument
    parser.add_argument("--which", "-w", help="Which onedrive to use? see: cat: ~/dotfiles/settings/paths.toml", default="default")
    parser.add_argument("--remote_dir", "-d", help="Remote directory to send to.", default="")
    parser.add_argument("--relative_to_home", "-R", help="Relative to `myhome` folder", action="store_true")  # default is False
    parser.add_argument("--key", "-k", help="Key for encryption", default=None)
    parser.add_argument("--pwd", "-p", help="Password for encryption", default=None)

    args = parser.parse_args()
    main(file=args.file, unzip=args.unzip, decrypt=args.decrypt, which=args.which, overwrite=args.overwrite, pwd=args.pwd, key=args.key)


if __name__ == "__main__":
    arg_parser()




"""
The executable of this program is suspicious according to virustotal.
"""

from machineconfig.utils.utils import get_latest_release, find_move_delete_linux
import crocodile.toolbox as tb
from platform import system


url = r'https://github.com/uw-labs/strongbox'


def main():
    d_url = get_latest_release(url)
    v = d_url.name.replace("v", "")
    if system() == 'Linux':
        f = d_url.joinpath(f"strongbox_{v}_linux_amd64").download().with_name("strongbox", inplace=True)
        find_move_delete_linux(f, 'tldr', delete=True)
        tb.Terminal().run("tldr --update")

    elif system() == 'Windows':
        f = d_url.joinpath(f"strongbox_{v}_windows_amd64.exe").download().with_name("strongbox.exe", inplace=True)
        f.move(folder=f.get_env().WindowsApps, overwrite=True)

    else:
        raise NotImplementedError(f"System {system()} not supported")
    tb.Terminal().run("strongbox -git-config", shell="powershell").print()


if __name__ == '__main__':
    main()




from crocodile.msc.odds import capture_from_webcam
import crocodile.toolbox as tb
import argparse


def main():
    parser = argparse.ArgumentParser(description='FTP client')
    parser.add_argument("--to_text", "-t", help="Send recursively.", action="store_true")  # default is False
    args = parser.parse_args()

    img_path = capture_from_webcam(show=False, wait=False, save=True)
    if args.to_text:
        q = tb.Terminal().run(f"cd ~/AppData/Local/Tesseract-OCR; pytesseract '{img_path}'", shell="pwsh").capture().op
        print(q)
    else:
        print(img_path)


if __name__ == '__main__':
    main()




# this is a plugin for OBS Studio to remove background from a video

from machineconfig.utils.utils import get_latest_release, tb


url = "https://github.com/royshil/obs-backgroundremoval"


def main(version=None):
    link = get_latest_release(url, download_n_extract=False, version=version)
    link = link.joinpath(f"obs-backgroundremoval-{link.parts[-1][1:]}-win64.zip").download()
    # link.search(r=True, folders=False).apply(lambda file: file.move(tb.P(fr"C:/Program Files/obs-studio").joinpath(file.relative_to(link)), overwrite=True))
    # Expand-Archive {link} -DestinationPath 'C:\Program Files\obs-studio\' -Force


if __name__ == '__main__':
    main()





"""
Not used because it is not mature enough.
--version doesn't work
"""

from machineconfig.utils.utils import get_latest_release, tb


url = 'https://github.com/gsass1/NTop'


def main():
    link = get_latest_release(url, download_n_extract=False)
    link.joinpath('ntop.exe').download().move(folder=tb.get_env().WindowsApps, name="htop.exe", overwrite=True)


if __name__ == '__main__':
    main()




import crocodile.toolbox as tb
from machineconfig.utils.utils import get_latest_release
from rich.console import Console
from platform import system


def main(version=None):
    if system() == "Windows":
        console = Console()
        console.rule("Installing diskonaut")
        url = r'https://drive.google.com/uc?id=1MsDaW6JXmS1LVyfThtUuJy4WRxODEjZB&export=download'
        dir_ = tb.P(url).download(name='diskonaut.zip').unzip(inplace=True)
        dir_.search()[0].move(folder=tb.get_env().WindowsApps, overwrite=True)
        dir_.delete(sure=True)
        console.rule("Completed Installation")
    else:
        url = r'https://github.com/imsnif/diskonaut'
        url = get_latest_release(url, download_n_extract=True, suffix="unknown-linux-musl", compression="tar.gz", linux=True, version=version)


if __name__ == '__main__':
    main()





from machineconfig.utils.utils import get_latest_release, find_move_delete_linux
import crocodile.toolbox as tb

# bash_integration = """
#
# # adopted from https://zellij.dev/documentation/integration.html
# # eval "$(zellij setup --generate-auto-start bash)"
# z_ls  # make sure it is towards the end of the script.
#
# """  # MOVED TO PATCHES


def main(version=None):
    _ = version
    _ = get_latest_release("https://github.com/zellij-org/zellij", download_n_extract=False, linux=True)
    download = tb.P(f"https://github.com/zellij-org/zellij/releases/latest/download/zellij-x86_64-unknown-linux-musl.tar.gz").download().ungz_untar(inplace=True)
    find_move_delete_linux(downloaded=download, tool_name="zellij")

    # profile_path = tb.P.home().joinpath(".bashrc")
    # profile_text = profile_path.read_text()
    # if bash_integration not in profile_text:
    #     profile_path.write_text(profile_text + bash_integration)


if __name__ == '__main__':
    main()





import crocodile.toolbox as tb
from platform import system
import machineconfig

# https://github.com/LunarVim/LunarVim


def main(version=None):
    _ = version
    if system() == "Windows":
        tb.P(r"AppData/Local/lvim").delete(sure=True)
        install_script = "$LV_BRANCH='release-1.2/neovim-0.8'; Invoke-WebRequest https://raw.githubusercontent.com/LunarVim/LunarVim/master/utils/installer/install.ps1 -UseBasicParsing | Invoke-Expression"
        # install_script = "Invoke-WebRequest    https://raw.githubusercontent.com/LunarVim/LunarVim/master/utils/installer/install.ps1 -UseBasicParsing | Invoke-Expression"
        uninstall_script = "Invoke-WebRequest https://raw.githubusercontent.com/LunarVim/LunarVim/master/utils/installer/uninstall.ps1 -UseBasicParsing | Invoke-Expression"
        script = f"{uninstall_script} ; {install_script}"
        # tb.T().run(script, shell="pwsh")
        # tb.P(r"AppData/Local/lvim").joinpath("config.lua").append_text(tb.P(machineconfig.__file__).joinpath("src/machineconfig/settings/lvim_windows/config_additional.lua").read_text())
        return script
    else:
        tb.P("~/.config/lvim").delete(sure=True)
        install_script = "LV_BRANCH='release-1.2/neovim-0.8' bash <(curl -s https://raw.githubusercontent.com/lunarvim/lunarvim/master/utils/installer/install.sh)"
        # install_script = r"bash <(curl -s https://raw.githubusercontent.com/LunarVim/LunarVim/rolling/utils/installer/install-neovim-from-release)"
        # script = install_script
        # tb.T().run(script, shell="pwsh")
        return install_script


if __name__ == '__main__':
    main()




import crocodile.toolbox as tb
import platform
# app_name = "something"
# tb.T().run(f"cargo install {app_name}")


def get_shell_script(url=r"https://github.com/atanunq/viu"):
    tool_name = url.split('/')[-1]

    # move command is not required since tool will go to .cargo/bin which is in PATH by default.
    # move_command = f"mv {exe} {tb.get_env().WindowsApps.as_posix()}/" if platform.platform() == "Windows" else f"sudo mv {exe} /usr/local/bin/"
    # {move_command}

    script = f"""
cd ~
git clone --depth 1 {url} 
cd {tool_name}
cargo install --path .
"""
    print(f"Executing {script}")
    if platform.system() == "Windows":
        tb.Terminal(stdout=None).run(f". {tb.P.tmpfile(suffix='.ps1').write_text(script)}", shell="pwsh").print()
    else:
        tb.Terminal(stdout=None).run(script, shell="pwsh")

    exe = tb.P.home().joinpath(f".cargo/bin/{tool_name}" + (".exe" if platform.system() == "Windows" else ""))

    try:
        tb.P.home().joinpath(tool_name).delete(sure=True)
    except PermissionError:
        print(f"PermissionError, couldn't delete: {tb.P.home().joinpath(tool_name)}")

    if platform.system() == "Windows":
        exe = exe.move(folder=tb.get_env().WindowsApps)
        share_link = exe.to_cloud("odg1", share=True)
        return share_link
    elif platform.system() == "Linux":
        tb.Terminal().run(f"sudo mv {exe} /usr/local/bin")
        exe = tb.P(r"/usr/local/bin").joinpath(exe.name)
        exe.to_cloud("odg1", share=True)
    else:
        raise NotImplementedError(f"Platform {platform.system()} not supported.")


# after cargo install diskonaut
# then mv ~/.cargo/bin/diskonaut.exe ~/AppData/Local/Microsoft/WindowsApps/
# then bu_gdrive_sx.ps1 .\diskonaut.exe -sRz  # zipping is vital to avoid security layers and keep file metadata.




from crocodile.file_management import P
from crocodile.environment import WIN_FROM_WSL, system, WSL_FROM_WIN, UserName
import argparse


def main():
    parser = argparse.ArgumentParser(description="""Move and copy files across between WSL & Windows."
The direction is automatically determined by sensing the execution environment.
Otherwise, a flag must be raised to indicate the direction.""")

    # positional argument
    parser.add_argument("path", help="path of file/folder to transfer over.")
    # FLAGS
    # this is dangerous and no gaurantee on no corruption.
    parser.add_argument("--same_file_system", "-s", help="Move file.", action="store_true")  # default is False
    # optional argument
    parser.add_argument("--destination", "-d", help="New path.", default="")
    parser.add_argument("--pwd", "-P", help="Password for encryption", default=None)
    parser.add_argument("--sshkey", "-i", help="path to ssh private key", default=None)
    parser.add_argument("--port", "-p", help="port number", default=None)
    parser.add_argument("--zip_first", "-z", help="Zip before transferring.", action="store_true")  # default is False

    args = parser.parse_args()
    path = P(args.path).expanduser().absolute()

    if args.same_file_system:
        if system == "Windows":  # move files over to WSL
            path.copy(folder=WSL_FROM_WIN.joinpath(UserName).joinpath(path.rel2home().parent), overwrite=True)  # the following works for files and folders alike.
        else:  # move files from WSL to win
            path.copy(folder=WIN_FROM_WSL.joinpath(UserName).joinpath(path.rel2home().parent), overwrite=True)
    else:
        from crocodile.meta import SSH
        ssh = SSH(port=args.port or (2222 if system == "Windows" else 22), sshkey=args.sshkey)
        ssh.copy_from_here(source=path, target=args.destination, z=args.zip_first)


if __name__ == '__main__':
    main()
